#!/usr/bin/env python3

import re
import os
import subprocess
from time import strftime
from MITRE_AT_template_creator import MITRE_AT_template

CAMPAIGNS = [
    "C0014",  # generate the MATT for the Wocao     campaign
    "C0022",  # generate the MATT for the Dream Job campaign
#   "C0018",
#    "",  # generate the complete MATT (all techniques and tactics)
]

DIFFICULTIES = [
    "hard",
    "easy",
    "default",
]

PATH = "../results/"


def template_name(difficulty: str, campaign: str) -> str:
    return f"MATT_{difficulty}_{campaign}"


print(">>  Creating templates  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
for campaign in CAMPAIGNS:
    for difficulty in DIFFICULTIES:
        print(f"\tCreating \"{difficulty}\" MATT for campaign {campaign}")
        AT = MITRE_AT_template(campaign, difficulty)
        MATT = template_name(difficulty, campaign) + ".py"
        with open(MATT, "w") as f:
            f.write(AT)
        print(f"\t- Written file {MATT}")
print("<<  DONE  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")

print(">>  Executing templates  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
for campaign in CAMPAIGNS:
    for difficulty in DIFFICULTIES:
        print(f"\tExecuting \"{difficulty}\" MATT for campaign {campaign}")
        MATT = template_name(difficulty, campaign) + ".py"
        result = subprocess.run(["python3", MATT], capture_output=True, text=True)
        with open(MATT[:-3] + ".out", "w") as f:
            f.write(result.stdout)
        with open(MATT[:-3] + ".err", "w") as f:
            f.write(result.stderr)
        # capture and print probability
        p = re.findall("cATM:[:blank:]*.+", result.stdout)[0][6:]
        print("\t- Probability of attack: {:.2e}".format(float(p)))
        print(f"\t- Written files {MATT[:-3]}.{{out,err}}")
print("<<  DONE  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")

print(">>  Storing results  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
results = PATH + "results_MATT_" + strftime("%b%d_%H%M%S")
subprocess.Popen(f"mkdir {results}", shell=True)
subprocess.Popen("sleep 1 ", shell=True)
subprocess.Popen(f"mv MATT_* {results}", shell=True)
# subprocess.Popen(f"ls {results}", shell=True)
print(f"\tResults are in {results}/")
print("<<  DONE  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
