#!/usr/bin/env python3

import pygraphviz
from AttackTree import AttackTree

AT = AttackTree()

campaign_tactech = {'collection': {'T1119', 'T1005', 'T1056.001', 'T1115', 'T1560.001', 'T1074.001'}, 'privilege-escalation': {'T1055.013', 'T1055.009', 'T1055.012', 'T1078.002', 'T1053.005', 'T1055.015', 'T1055.001', 'T1055.014', 'T1055.008', 'T1055.002', 'T1055.003', 'T1055.005', 'T1055.011', 'T1055.004', 'T1078.003'}, 'command-and-control': {'T1105', 'T1001.001', 'T1071.001', 'T1095', 'T1001.003', 'T1090.001', 'T1001.002', 'T1571', 'T1573.002', 'T1090.003'}, 'reconnaissance': {'T1589.001', 'T1589.003', 'T1589.002'}, 'initial-access': {'T1078.003', 'T1190', 'T1133', 'T1078.002'}, 'resource-development': {'T1587.001', 'T1588.002', 'T1585.002', 'T1583.004'}, 'lateral-movement': {'T1570', 'T1021.002'}, 'credential-access': {'T1003.001', 'T1003.006', 'T1555.005', 'T1558.003', 'T1552.004', 'T1056.001', 'T1111'}, 'defense-evasion': {'T1055.012', 'T1027.005', 'T1070.001', 'T1036.005', 'T1055.002', 'T1055.004', 'T1078.003', 'T1562.004', 'T1055.014', 'T1055.005', 'T1055.013', 'T1055.009', 'T1055.015', 'T1055.001', 'T1027.010', 'T1055.008', 'T1055.003', 'T1112', 'T1055.011', 'T1070.004', 'T1078.002'}, 'execution': {'T1047', 'T1053.005', 'T1059.006', 'T1106', 'T1059.005', 'T1569.002', 'T1059.003', 'T1059.001'}, 'exfiltration': {'T1041'}, 'discovery': {'T1046', 'T1033', 'T1057', 'T1012', 'T1135', 'T1518.001', 'T1016.001', 'T1087.002', 'T1124', 'T1082', 'T1083', 'T1049', 'T1018', 'T1007', 'T1069.001', 'T1120'}, 'persistence': {'T1078.002', 'T1133', 'T1505.003', 'T1053.005', 'T1078.003'}, 'impact': set()}

# - - - - - - - - - - - - - - - - - -
# Tactic: collection

# Data from Configuration Repository
BASes_collection_DatafromConfigurationRepository = [
	('Data from Configuration Repository: Network Device Configuration Dump', 'collection', 'T1602.002'),
	('Data from Configuration Repository: SNMP (MIB Dump)', 'collection', 'T1602.001'),
]
for i in range(len(BASes_collection_DatafromConfigurationRepository)):
	if BASes_collection_DatafromConfigurationRepository[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_DatafromConfigurationRepository[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_DatafromConfigurationRepository[i], is_dummy=True)

AT.add_gate_logic("collection_DatafromConfigurationRepository", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_DatafromConfigurationRepository])

# Adversary-in-the-Middle
BASes_collection_AdversaryintheMiddle = [
	('Adversary-in-the-Middle: ARP Cache Poisoning', 'collection', 'T1557.002'),
	('Adversary-in-the-Middle: DHCP Spoofing', 'collection', 'T1557.003'),
	('Adversary-in-the-Middle: LLMNR/NBT-NS Poisoning and SMB Relay', 'collection', 'T1557.001'),
]
for i in range(len(BASes_collection_AdversaryintheMiddle)):
	if BASes_collection_AdversaryintheMiddle[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_AdversaryintheMiddle[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_AdversaryintheMiddle[i], is_dummy=True)

AT.add_gate_logic("collection_AdversaryintheMiddle", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_AdversaryintheMiddle])

# Email Collection
BASes_collection_EmailCollection = [
	('Email Collection: Email Forwarding Rule', 'collection', 'T1114.003'),
	('Email Collection: Local Email Collection', 'collection', 'T1114.001'),
	('Email Collection: Remote Email Collection', 'collection', 'T1114.002'),
]
for i in range(len(BASes_collection_EmailCollection)):
	if BASes_collection_EmailCollection[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_EmailCollection[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_EmailCollection[i], is_dummy=True)

AT.add_gate_logic("collection_EmailCollection", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_EmailCollection])

# Input Capture
BASes_collection_InputCapture = [
	('Input Capture: Credential API Hooking', 'collection', 'T1056.004'),
	('Input Capture: GUI Input Capture', 'collection', 'T1056.002'),
	('Input Capture: Keylogging', 'collection', 'T1056.001'),
	('Input Capture: Web Portal Capture', 'collection', 'T1056.003'),
]
for i in range(len(BASes_collection_InputCapture)):
	if BASes_collection_InputCapture[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_InputCapture[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_InputCapture[i], is_dummy=True)

AT.add_gate_logic("collection_InputCapture", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_InputCapture])

# Archive Collected Data
BASes_collection_ArchiveCollectedData = [
	('Archive Collected Data: Archive via Custom Method', 'collection', 'T1560.003'),
	('Archive Collected Data: Archive via Library', 'collection', 'T1560.002'),
	('Archive Collected Data: Archive via Utility', 'collection', 'T1560.001'),
]
for i in range(len(BASes_collection_ArchiveCollectedData)):
	if BASes_collection_ArchiveCollectedData[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_ArchiveCollectedData[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_ArchiveCollectedData[i], is_dummy=True)

AT.add_gate_logic("collection_ArchiveCollectedData", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_ArchiveCollectedData])

# Data Staged
BASes_collection_DataStaged = [
	('Data Staged: Local Data Staging', 'collection', 'T1074.001'),
	('Data Staged: Remote Data Staging', 'collection', 'T1074.002'),
]
for i in range(len(BASes_collection_DataStaged)):
	if BASes_collection_DataStaged[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_DataStaged[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_DataStaged[i], is_dummy=True)

AT.add_gate_logic("collection_DataStaged", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_DataStaged])

# Data from Information Repositories
BASes_collection_DatafromInformationRepositories = [
	('Data from Information Repositories: Code Repositories', 'collection', 'T1213.003'),
	('Data from Information Repositories: Confluence', 'collection', 'T1213.001'),
	('Data from Information Repositories: Sharepoint', 'collection', 'T1213.002'),
]
for i in range(len(BASes_collection_DatafromInformationRepositories)):
	if BASes_collection_DatafromInformationRepositories[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_DatafromInformationRepositories[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_DatafromInformationRepositories[i], is_dummy=True)

AT.add_gate_logic("collection_DatafromInformationRepositories", "OR", color="#88ccee", children=[ x[0] for x in BASes_collection_DatafromInformationRepositories])

# direct BASes
BASes_collection_childless = [
	('Audio Capture', 'collection', 'T1123'),
	('Automated Collection', 'collection', 'T1119'),
	('Browser Session Hijacking', 'collection', 'T1185'),
	('Clipboard Data', 'collection', 'T1115'),
	('Data from Cloud Storage', 'collection', 'T1530'),
	('Data from Local System', 'collection', 'T1005'),
	('Data from Network Shared Drive', 'collection', 'T1039'),
	('Data from Removable Media', 'collection', 'T1025'),
	('Screen Capture', 'collection', 'T1113'),
	('Video Capture', 'collection', 'T1125'),
]
for i in range(len(BASes_collection_childless)):
	if BASes_collection_childless[i][-1] in campaign_tactech["collection"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_collection_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_collection_childless[i], is_dummy=True)

AT.add_gate_logic("collection", "AND", color="#cc79a7", children=["collection_DatafromConfigurationRepository","collection_AdversaryintheMiddle","collection_EmailCollection","collection_InputCapture","collection_ArchiveCollectedData","collection_DataStaged","collection_DatafromInformationRepositories"]+[x[0] for x in BASes_collection_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: command-and-control

# Data Encoding
BASes_commandandcontrol_DataEncoding = [
	('Data Encoding: Non-Standard Encoding', 'command-and-control', 'T1132.002'),
	('Data Encoding: Standard Encoding', 'command-and-control', 'T1132.001'),
]
for i in range(len(BASes_commandandcontrol_DataEncoding)):
	if BASes_commandandcontrol_DataEncoding[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_DataEncoding[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_DataEncoding[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_DataEncoding", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_DataEncoding])

# Data Obfuscation
BASes_commandandcontrol_DataObfuscation = [
	('Data Obfuscation: Junk Data', 'command-and-control', 'T1001.001'),
	('Data Obfuscation: Protocol Impersonation', 'command-and-control', 'T1001.003'),
	('Data Obfuscation: Steganography', 'command-and-control', 'T1001.002'),
]
for i in range(len(BASes_commandandcontrol_DataObfuscation)):
	if BASes_commandandcontrol_DataObfuscation[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_DataObfuscation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_DataObfuscation[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_DataObfuscation", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_DataObfuscation])

# Proxy
BASes_commandandcontrol_Proxy = [
	('Proxy: Domain Fronting', 'command-and-control', 'T1090.004'),
	('Proxy: External Proxy', 'command-and-control', 'T1090.002'),
	('Proxy: Internal Proxy', 'command-and-control', 'T1090.001'),
	('Proxy: Multi-hop Proxy', 'command-and-control', 'T1090.003'),
]
for i in range(len(BASes_commandandcontrol_Proxy)):
	if BASes_commandandcontrol_Proxy[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_Proxy[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_Proxy[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_Proxy", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_Proxy])

# Application Layer Protocol
BASes_commandandcontrol_ApplicationLayerProtocol = [
	('Application Layer Protocol: DNS', 'command-and-control', 'T1071.004'),
	('Application Layer Protocol: File Transfer Protocols', 'command-and-control', 'T1071.002'),
	('Application Layer Protocol: Mail Protocols', 'command-and-control', 'T1071.003'),
	('Application Layer Protocol: Web Protocols', 'command-and-control', 'T1071.001'),
]
for i in range(len(BASes_commandandcontrol_ApplicationLayerProtocol)):
	if BASes_commandandcontrol_ApplicationLayerProtocol[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_ApplicationLayerProtocol[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_ApplicationLayerProtocol[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_ApplicationLayerProtocol", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_ApplicationLayerProtocol])

# Web Service
BASes_commandandcontrol_WebService = [
	('Web Service: Bidirectional Communication', 'command-and-control', 'T1102.002'),
	('Web Service: Dead Drop Resolver', 'command-and-control', 'T1102.001'),
	('Web Service: One-Way Communication', 'command-and-control', 'T1102.003'),
]
for i in range(len(BASes_commandandcontrol_WebService)):
	if BASes_commandandcontrol_WebService[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_WebService[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_WebService[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_WebService", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_WebService])

# Traffic Signaling
BASes_commandandcontrol_TrafficSignaling = [
	('Traffic Signaling: Port Knocking', 'command-and-control', 'T1205.001'),
	('Traffic Signaling: Socket Filters', 'command-and-control', 'T1205.002'),
]
for i in range(len(BASes_commandandcontrol_TrafficSignaling)):
	if BASes_commandandcontrol_TrafficSignaling[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_TrafficSignaling[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_TrafficSignaling[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_TrafficSignaling", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_TrafficSignaling])

# Dynamic Resolution
BASes_commandandcontrol_DynamicResolution = [
	('Dynamic Resolution: DNS Calculation', 'command-and-control', 'T1568.003'),
	('Dynamic Resolution: Domain Generation Algorithms', 'command-and-control', 'T1568.002'),
	('Dynamic Resolution: Fast Flux DNS', 'command-and-control', 'T1568.001'),
]
for i in range(len(BASes_commandandcontrol_DynamicResolution)):
	if BASes_commandandcontrol_DynamicResolution[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_DynamicResolution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_DynamicResolution[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_DynamicResolution", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_DynamicResolution])

# Encrypted Channel
BASes_commandandcontrol_EncryptedChannel = [
	('Encrypted Channel: Asymmetric Cryptography', 'command-and-control', 'T1573.002'),
	('Encrypted Channel: Symmetric Cryptography', 'command-and-control', 'T1573.001'),
]
for i in range(len(BASes_commandandcontrol_EncryptedChannel)):
	if BASes_commandandcontrol_EncryptedChannel[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_EncryptedChannel[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_EncryptedChannel[i], is_dummy=True)

AT.add_gate_logic("commandandcontrol_EncryptedChannel", "OR", color="#88ccee", children=[ x[0] for x in BASes_commandandcontrol_EncryptedChannel])

# direct BASes
BASes_commandandcontrol_childless = [
	('Communication Through Removable Media', 'command-and-control', 'T1092'),
	('Content Injection', 'command-and-control', 'T1659'),
	('Fallback Channels', 'command-and-control', 'T1008'),
	('Ingress Tool Transfer', 'command-and-control', 'T1105'),
	('Multi-Stage Channels', 'command-and-control', 'T1104'),
	('Non-Application Layer Protocol', 'command-and-control', 'T1095'),
	('Non-Standard Port', 'command-and-control', 'T1571'),
	('Protocol Tunneling', 'command-and-control', 'T1572'),
	('Remote Access Software', 'command-and-control', 'T1219'),
]
for i in range(len(BASes_commandandcontrol_childless)):
	if BASes_commandandcontrol_childless[i][-1] in campaign_tactech["command-and-control"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_commandandcontrol_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_commandandcontrol_childless[i], is_dummy=True)

AT.add_gate_logic("command-and-control", "AND", color="#cc79a7", children=["commandandcontrol_DataEncoding","commandandcontrol_DataObfuscation","commandandcontrol_Proxy","commandandcontrol_ApplicationLayerProtocol","commandandcontrol_WebService","commandandcontrol_TrafficSignaling","commandandcontrol_DynamicResolution","commandandcontrol_EncryptedChannel"]+[x[0] for x in BASes_commandandcontrol_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: reconnaissance

# Search Open Technical Databases
BASes_reconnaissance_SearchOpenTechnicalDatabases = [
	('Search Open Technical Databases: CDNs', 'reconnaissance', 'T1596.004'),
	('Search Open Technical Databases: DNS/Passive DNS', 'reconnaissance', 'T1596.001'),
	('Search Open Technical Databases: Digital Certificates', 'reconnaissance', 'T1596.003'),
	('Search Open Technical Databases: Scan Databases', 'reconnaissance', 'T1596.005'),
	('Search Open Technical Databases: WHOIS', 'reconnaissance', 'T1596.002'),
]
for i in range(len(BASes_reconnaissance_SearchOpenTechnicalDatabases)):
	if BASes_reconnaissance_SearchOpenTechnicalDatabases[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_SearchOpenTechnicalDatabases[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_SearchOpenTechnicalDatabases[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_SearchOpenTechnicalDatabases", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_SearchOpenTechnicalDatabases])

# Search Open Websites/Domains
BASes_reconnaissance_SearchOpenWebsitesDomains = [
	('Search Open Websites/Domains: Code Repositories', 'reconnaissance', 'T1593.003'),
	('Search Open Websites/Domains: Search Engines', 'reconnaissance', 'T1593.002'),
	('Search Open Websites/Domains: Social Media', 'reconnaissance', 'T1593.001'),
]
for i in range(len(BASes_reconnaissance_SearchOpenWebsitesDomains)):
	if BASes_reconnaissance_SearchOpenWebsitesDomains[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_SearchOpenWebsitesDomains[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_SearchOpenWebsitesDomains[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_SearchOpenWebsitesDomains", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_SearchOpenWebsitesDomains])

# Gather Victim Org Information
BASes_reconnaissance_GatherVictimOrgInformation = [
	('Gather Victim Org Information: Business Relationships', 'reconnaissance', 'T1591.002'),
	('Gather Victim Org Information: Determine Physical Locations', 'reconnaissance', 'T1591.001'),
	('Gather Victim Org Information: Identify Business Tempo', 'reconnaissance', 'T1591.003'),
	('Gather Victim Org Information: Identify Roles', 'reconnaissance', 'T1591.004'),
]
for i in range(len(BASes_reconnaissance_GatherVictimOrgInformation)):
	if BASes_reconnaissance_GatherVictimOrgInformation[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimOrgInformation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimOrgInformation[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_GatherVictimOrgInformation", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_GatherVictimOrgInformation])

# Gather Victim Network Information
BASes_reconnaissance_GatherVictimNetworkInformation = [
	('Gather Victim Network Information: DNS', 'reconnaissance', 'T1590.002'),
	('Gather Victim Network Information: Domain Properties', 'reconnaissance', 'T1590.001'),
	('Gather Victim Network Information: IP Addresses', 'reconnaissance', 'T1590.005'),
	('Gather Victim Network Information: Network Security Appliances', 'reconnaissance', 'T1590.006'),
	('Gather Victim Network Information: Network Topology', 'reconnaissance', 'T1590.004'),
	('Gather Victim Network Information: Network Trust Dependencies', 'reconnaissance', 'T1590.003'),
]
for i in range(len(BASes_reconnaissance_GatherVictimNetworkInformation)):
	if BASes_reconnaissance_GatherVictimNetworkInformation[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimNetworkInformation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimNetworkInformation[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_GatherVictimNetworkInformation", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_GatherVictimNetworkInformation])

# Search Closed Sources
BASes_reconnaissance_SearchClosedSources = [
	('Search Closed Sources: Purchase Technical Data', 'reconnaissance', 'T1597.002'),
	('Search Closed Sources: Threat Intel Vendors', 'reconnaissance', 'T1597.001'),
]
for i in range(len(BASes_reconnaissance_SearchClosedSources)):
	if BASes_reconnaissance_SearchClosedSources[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_SearchClosedSources[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_SearchClosedSources[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_SearchClosedSources", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_SearchClosedSources])

# Gather Victim Host Information
BASes_reconnaissance_GatherVictimHostInformation = [
	('Gather Victim Host Information: Client Configurations', 'reconnaissance', 'T1592.004'),
	('Gather Victim Host Information: Firmware', 'reconnaissance', 'T1592.003'),
	('Gather Victim Host Information: Hardware', 'reconnaissance', 'T1592.001'),
	('Gather Victim Host Information: Software', 'reconnaissance', 'T1592.002'),
]
for i in range(len(BASes_reconnaissance_GatherVictimHostInformation)):
	if BASes_reconnaissance_GatherVictimHostInformation[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimHostInformation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimHostInformation[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_GatherVictimHostInformation", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_GatherVictimHostInformation])

# Phishing for Information
BASes_reconnaissance_PhishingforInformation = [
	('Phishing for Information: Spearphishing Attachment', 'reconnaissance', 'T1598.002'),
	('Phishing for Information: Spearphishing Link', 'reconnaissance', 'T1598.003'),
	('Phishing for Information: Spearphishing Service', 'reconnaissance', 'T1598.001'),
	('Phishing for Information: Spearphishing Voice', 'reconnaissance', 'T1598.004'),
]
for i in range(len(BASes_reconnaissance_PhishingforInformation)):
	if BASes_reconnaissance_PhishingforInformation[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_PhishingforInformation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_PhishingforInformation[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_PhishingforInformation", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_PhishingforInformation])

# Gather Victim Identity Information
BASes_reconnaissance_GatherVictimIdentityInformation = [
	('Gather Victim Identity Information: Credentials', 'reconnaissance', 'T1589.001'),
	('Gather Victim Identity Information: Email Addresses', 'reconnaissance', 'T1589.002'),
	('Gather Victim Identity Information: Employee Names', 'reconnaissance', 'T1589.003'),
]
for i in range(len(BASes_reconnaissance_GatherVictimIdentityInformation)):
	if BASes_reconnaissance_GatherVictimIdentityInformation[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimIdentityInformation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_GatherVictimIdentityInformation[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_GatherVictimIdentityInformation", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_GatherVictimIdentityInformation])

# Active Scanning
BASes_reconnaissance_ActiveScanning = [
	('Active Scanning: Scanning IP Blocks', 'reconnaissance', 'T1595.001'),
	('Active Scanning: Vulnerability Scanning', 'reconnaissance', 'T1595.002'),
	('Active Scanning: Wordlist Scanning', 'reconnaissance', 'T1595.003'),
]
for i in range(len(BASes_reconnaissance_ActiveScanning)):
	if BASes_reconnaissance_ActiveScanning[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_ActiveScanning[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_ActiveScanning[i], is_dummy=True)

AT.add_gate_logic("reconnaissance_ActiveScanning", "OR", color="#88ccee", children=[ x[0] for x in BASes_reconnaissance_ActiveScanning])

# direct BASes
BASes_reconnaissance_childless = [
	('Search Victim-Owned Websites', 'reconnaissance', 'T1594'),
]
for i in range(len(BASes_reconnaissance_childless)):
	if BASes_reconnaissance_childless[i][-1] in campaign_tactech["reconnaissance"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_reconnaissance_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_reconnaissance_childless[i], is_dummy=True)

AT.add_gate_logic("reconnaissance", "AND", color="#cc79a7", children=["reconnaissance_SearchOpenTechnicalDatabases","reconnaissance_SearchOpenWebsitesDomains","reconnaissance_GatherVictimOrgInformation","reconnaissance_GatherVictimNetworkInformation","reconnaissance_SearchClosedSources","reconnaissance_GatherVictimHostInformation","reconnaissance_PhishingforInformation","reconnaissance_GatherVictimIdentityInformation","reconnaissance_ActiveScanning"]+[x[0] for x in BASes_reconnaissance_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: discovery

# Account Discovery
BASes_discovery_AccountDiscovery = [
	('Account Discovery: Cloud Account', 'discovery', 'T1087.004'),
	('Account Discovery: Domain Account', 'discovery', 'T1087.002'),
	('Account Discovery: Email Account', 'discovery', 'T1087.003'),
	('Account Discovery: Local Account', 'discovery', 'T1087.001'),
]
for i in range(len(BASes_discovery_AccountDiscovery)):
	if BASes_discovery_AccountDiscovery[i][-1] in campaign_tactech["discovery"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_discovery_AccountDiscovery[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_discovery_AccountDiscovery[i], is_dummy=True)

AT.add_gate_logic("discovery_AccountDiscovery", "OR", color="#88ccee", children=[ x[0] for x in BASes_discovery_AccountDiscovery])

# Permission Groups Discovery
BASes_discovery_PermissionGroupsDiscovery = [
	('Permission Groups Discovery: Cloud Groups', 'discovery', 'T1069.003'),
	('Permission Groups Discovery: Domain Groups', 'discovery', 'T1069.002'),
	('Permission Groups Discovery: Local Groups', 'discovery', 'T1069.001'),
]
for i in range(len(BASes_discovery_PermissionGroupsDiscovery)):
	if BASes_discovery_PermissionGroupsDiscovery[i][-1] in campaign_tactech["discovery"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_discovery_PermissionGroupsDiscovery[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_discovery_PermissionGroupsDiscovery[i], is_dummy=True)

AT.add_gate_logic("discovery_PermissionGroupsDiscovery", "OR", color="#88ccee", children=[ x[0] for x in BASes_discovery_PermissionGroupsDiscovery])

# Virtualization/Sandbox Evasion
BASes_discovery_VirtualizationSandboxEvasion = [
	('Virtualization/Sandbox Evasion: System Checks', 'discovery', 'T1497.001'),
	('Virtualization/Sandbox Evasion: Time Based Evasion', 'discovery', 'T1497.003'),
	('Virtualization/Sandbox Evasion: User Activity Based Checks', 'discovery', 'T1497.002'),
]
for i in range(len(BASes_discovery_VirtualizationSandboxEvasion)):
	if BASes_discovery_VirtualizationSandboxEvasion[i][-1] in campaign_tactech["discovery"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_discovery_VirtualizationSandboxEvasion[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_discovery_VirtualizationSandboxEvasion[i], is_dummy=True)

AT.add_gate_logic("discovery_VirtualizationSandboxEvasion", "OR", color="#88ccee", children=[ x[0] for x in BASes_discovery_VirtualizationSandboxEvasion])

# System Network Configuration Discovery
BASes_discovery_SystemNetworkConfigurationDiscovery = [
	('System Network Configuration Discovery: Internet Connection Discovery', 'discovery', 'T1016.001'),
	('System Network Configuration Discovery: Wi-Fi Discovery', 'discovery', 'T1016.002'),
]
for i in range(len(BASes_discovery_SystemNetworkConfigurationDiscovery)):
	if BASes_discovery_SystemNetworkConfigurationDiscovery[i][-1] in campaign_tactech["discovery"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_discovery_SystemNetworkConfigurationDiscovery[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_discovery_SystemNetworkConfigurationDiscovery[i], is_dummy=True)

AT.add_gate_logic("discovery_SystemNetworkConfigurationDiscovery", "OR", color="#88ccee", children=[ x[0] for x in BASes_discovery_SystemNetworkConfigurationDiscovery])

# direct BASes
BASes_discovery_childless = [
	('Application Window Discovery', 'discovery', 'T1010'),
	('Browser Information Discovery', 'discovery', 'T1217'),
	('Cloud Infrastructure Discovery', 'discovery', 'T1580'),
	('Cloud Service Dashboard', 'discovery', 'T1538'),
	('Cloud Service Discovery', 'discovery', 'T1526'),
	('Cloud Storage Object Discovery', 'discovery', 'T1619'),
	('Container and Resource Discovery', 'discovery', 'T1613'),
	('Debugger Evasion', 'discovery', 'T1622'),
	('Device Driver Discovery', 'discovery', 'T1652'),
	('Domain Trust Discovery', 'discovery', 'T1482'),
	('File and Directory Discovery', 'discovery', 'T1083'),
	('Group Policy Discovery', 'discovery', 'T1615'),
	('Log Enumeration', 'discovery', 'T1654'),
	('Network Service Discovery', 'discovery', 'T1046'),
	('Network Share Discovery', 'discovery', 'T1135'),
	('Network Sniffing', 'discovery', 'T1040'),
	('Password Policy Discovery', 'discovery', 'T1201'),
	('Peripheral Device Discovery', 'discovery', 'T1120'),
	('Process Discovery', 'discovery', 'T1057'),
	('Query Registry', 'discovery', 'T1012'),
	('Remote System Discovery', 'discovery', 'T1018'),
	('Software Discovery: Security Software Discovery', 'discovery', 'T1518.001'),
	('System Information Discovery', 'discovery', 'T1082'),
	('System Location Discovery: System Language Discovery', 'discovery', 'T1614.001'),
	('System Network Connections Discovery', 'discovery', 'T1049'),
	('System Owner/User Discovery', 'discovery', 'T1033'),
	('System Service Discovery', 'discovery', 'T1007'),
	('System Time Discovery', 'discovery', 'T1124'),
]
for i in range(len(BASes_discovery_childless)):
	if BASes_discovery_childless[i][-1] in campaign_tactech["discovery"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_discovery_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_discovery_childless[i], is_dummy=True)

AT.add_gate_logic("discovery", "AND", color="#cc79a7", children=["discovery_AccountDiscovery","discovery_PermissionGroupsDiscovery","discovery_VirtualizationSandboxEvasion","discovery_SystemNetworkConfigurationDiscovery"]+[x[0] for x in BASes_discovery_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: initial-access

# Supply Chain Compromise
BASes_initialaccess_SupplyChainCompromise = [
	('Supply Chain Compromise: Compromise Hardware Supply Chain', 'initial-access', 'T1195.003'),
	('Supply Chain Compromise: Compromise Software Dependencies and Development Tools', 'initial-access', 'T1195.001'),
	('Supply Chain Compromise: Compromise Software Supply Chain', 'initial-access', 'T1195.002'),
]
for i in range(len(BASes_initialaccess_SupplyChainCompromise)):
	if BASes_initialaccess_SupplyChainCompromise[i][-1] in campaign_tactech["initial-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_initialaccess_SupplyChainCompromise[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_initialaccess_SupplyChainCompromise[i], is_dummy=True)

AT.add_gate_logic("initialaccess_SupplyChainCompromise", "OR", color="#88ccee", children=[ x[0] for x in BASes_initialaccess_SupplyChainCompromise])

# Phishing
BASes_initialaccess_Phishing = [
	('Phishing: Spearphishing Attachment', 'initial-access', 'T1566.001'),
	('Phishing: Spearphishing Link', 'initial-access', 'T1566.002'),
	('Phishing: Spearphishing Voice', 'initial-access', 'T1566.004'),
	('Phishing: Spearphishing via Service', 'initial-access', 'T1566.003'),
]
for i in range(len(BASes_initialaccess_Phishing)):
	if BASes_initialaccess_Phishing[i][-1] in campaign_tactech["initial-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_initialaccess_Phishing[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_initialaccess_Phishing[i], is_dummy=True)

AT.add_gate_logic("initialaccess_Phishing", "OR", color="#88ccee", children=[ x[0] for x in BASes_initialaccess_Phishing])

# Valid Accounts
BASes_initialaccess_ValidAccounts = [
	('Valid Accounts: Cloud Accounts', 'initial-access', 'T1078.004'),
	('Valid Accounts: Default Accounts', 'initial-access', 'T1078.001'),
	('Valid Accounts: Domain Accounts', 'initial-access', 'T1078.002'),
	('Valid Accounts: Local Accounts', 'initial-access', 'T1078.003'),
]
for i in range(len(BASes_initialaccess_ValidAccounts)):
	if BASes_initialaccess_ValidAccounts[i][-1] in campaign_tactech["initial-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_initialaccess_ValidAccounts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_initialaccess_ValidAccounts[i], is_dummy=True)

AT.add_gate_logic("initialaccess_ValidAccounts", "OR", color="#88ccee", children=[ x[0] for x in BASes_initialaccess_ValidAccounts])

# direct BASes
BASes_initialaccess_childless = [
	('Content Injection', 'initial-access', 'T1659'),
	('Drive-by Compromise', 'initial-access', 'T1189'),
	('Exploit Public-Facing Application', 'initial-access', 'T1190'),
	('External Remote Services', 'initial-access', 'T1133'),
	('Hardware Additions', 'initial-access', 'T1200'),
	('Replication Through Removable Media', 'initial-access', 'T1091'),
	('Trusted Relationship', 'initial-access', 'T1199'),
]
for i in range(len(BASes_initialaccess_childless)):
	if BASes_initialaccess_childless[i][-1] in campaign_tactech["initial-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_initialaccess_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_initialaccess_childless[i], is_dummy=True)

AT.add_gate_logic("initial-access", "AND", color="#cc79a7", children=["initialaccess_SupplyChainCompromise","initialaccess_Phishing","initialaccess_ValidAccounts"]+[x[0] for x in BASes_initialaccess_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: resource-development

# Compromise Infrastructure
BASes_resourcedevelopment_CompromiseInfrastructure = [
	('Compromise Infrastructure: Botnet', 'resource-development', 'T1584.005'),
	('Compromise Infrastructure: DNS Server', 'resource-development', 'T1584.002'),
	('Compromise Infrastructure: Domains', 'resource-development', 'T1584.001'),
	('Compromise Infrastructure: Server', 'resource-development', 'T1584.004'),
	('Compromise Infrastructure: Serverless', 'resource-development', 'T1584.007'),
	('Compromise Infrastructure: Virtual Private Server', 'resource-development', 'T1584.003'),
	('Compromise Infrastructure: Web Services', 'resource-development', 'T1584.006'),
]
for i in range(len(BASes_resourcedevelopment_CompromiseInfrastructure)):
	if BASes_resourcedevelopment_CompromiseInfrastructure[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_CompromiseInfrastructure[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_CompromiseInfrastructure[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_CompromiseInfrastructure", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_CompromiseInfrastructure])

# Establish Accounts
BASes_resourcedevelopment_EstablishAccounts = [
	('Establish Accounts: Cloud Accounts', 'resource-development', 'T1585.003'),
	('Establish Accounts: Email Accounts', 'resource-development', 'T1585.002'),
	('Establish Accounts: Social Media Accounts', 'resource-development', 'T1585.001'),
]
for i in range(len(BASes_resourcedevelopment_EstablishAccounts)):
	if BASes_resourcedevelopment_EstablishAccounts[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_EstablishAccounts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_EstablishAccounts[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_EstablishAccounts", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_EstablishAccounts])

# Compromise Accounts
BASes_resourcedevelopment_CompromiseAccounts = [
	('Compromise Accounts: Cloud Accounts', 'resource-development', 'T1586.003'),
	('Compromise Accounts: Email Accounts', 'resource-development', 'T1586.002'),
	('Compromise Accounts: Social Media Accounts', 'resource-development', 'T1586.001'),
]
for i in range(len(BASes_resourcedevelopment_CompromiseAccounts)):
	if BASes_resourcedevelopment_CompromiseAccounts[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_CompromiseAccounts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_CompromiseAccounts[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_CompromiseAccounts", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_CompromiseAccounts])

# Develop Capabilities
BASes_resourcedevelopment_DevelopCapabilities = [
	('Develop Capabilities: Code Signing Certificates', 'resource-development', 'T1587.002'),
	('Develop Capabilities: Digital Certificates', 'resource-development', 'T1587.003'),
	('Develop Capabilities: Exploits', 'resource-development', 'T1587.004'),
	('Develop Capabilities: Malware', 'resource-development', 'T1587.001'),
]
for i in range(len(BASes_resourcedevelopment_DevelopCapabilities)):
	if BASes_resourcedevelopment_DevelopCapabilities[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_DevelopCapabilities[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_DevelopCapabilities[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_DevelopCapabilities", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_DevelopCapabilities])

# Acquire Infrastructure
BASes_resourcedevelopment_AcquireInfrastructure = [
	('Acquire Infrastructure: Botnet', 'resource-development', 'T1583.005'),
	('Acquire Infrastructure: DNS Server', 'resource-development', 'T1583.002'),
	('Acquire Infrastructure: Domains', 'resource-development', 'T1583.001'),
	('Acquire Infrastructure: Malvertising', 'resource-development', 'T1583.008'),
	('Acquire Infrastructure: Server', 'resource-development', 'T1583.004'),
	('Acquire Infrastructure: Serverless', 'resource-development', 'T1583.007'),
	('Acquire Infrastructure: Virtual Private Server', 'resource-development', 'T1583.003'),
	('Acquire Infrastructure: Web Services', 'resource-development', 'T1583.006'),
]
for i in range(len(BASes_resourcedevelopment_AcquireInfrastructure)):
	if BASes_resourcedevelopment_AcquireInfrastructure[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_AcquireInfrastructure[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_AcquireInfrastructure[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_AcquireInfrastructure", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_AcquireInfrastructure])

# Obtain Capabilities
BASes_resourcedevelopment_ObtainCapabilities = [
	('Obtain Capabilities: Code Signing Certificates', 'resource-development', 'T1588.003'),
	('Obtain Capabilities: Digital Certificates', 'resource-development', 'T1588.004'),
	('Obtain Capabilities: Exploits', 'resource-development', 'T1588.005'),
	('Obtain Capabilities: Malware', 'resource-development', 'T1588.001'),
	('Obtain Capabilities: Tool', 'resource-development', 'T1588.002'),
	('Obtain Capabilities: Vulnerabilities', 'resource-development', 'T1588.006'),
]
for i in range(len(BASes_resourcedevelopment_ObtainCapabilities)):
	if BASes_resourcedevelopment_ObtainCapabilities[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_ObtainCapabilities[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_ObtainCapabilities[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_ObtainCapabilities", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_ObtainCapabilities])

# Stage Capabilities
BASes_resourcedevelopment_StageCapabilities = [
	('Stage Capabilities: Drive-by Target', 'resource-development', 'T1608.004'),
	('Stage Capabilities: Install Digital Certificate', 'resource-development', 'T1608.003'),
	('Stage Capabilities: Link Target', 'resource-development', 'T1608.005'),
	('Stage Capabilities: SEO Poisoning', 'resource-development', 'T1608.006'),
	('Stage Capabilities: Upload Malware', 'resource-development', 'T1608.001'),
	('Stage Capabilities: Upload Tool', 'resource-development', 'T1608.002'),
]
for i in range(len(BASes_resourcedevelopment_StageCapabilities)):
	if BASes_resourcedevelopment_StageCapabilities[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_StageCapabilities[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_StageCapabilities[i], is_dummy=True)

AT.add_gate_logic("resourcedevelopment_StageCapabilities", "OR", color="#88ccee", children=[ x[0] for x in BASes_resourcedevelopment_StageCapabilities])

# direct BASes
BASes_resourcedevelopment_childless = [
	('Acquire Access', 'resource-development', 'T1650'),
]
for i in range(len(BASes_resourcedevelopment_childless)):
	if BASes_resourcedevelopment_childless[i][-1] in campaign_tactech["resource-development"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_resourcedevelopment_childless[i], is_dummy=True)

AT.add_gate_logic("resource-development", "AND", color="#cc79a7", children=["resourcedevelopment_CompromiseInfrastructure","resourcedevelopment_EstablishAccounts","resourcedevelopment_CompromiseAccounts","resourcedevelopment_DevelopCapabilities","resourcedevelopment_AcquireInfrastructure","resourcedevelopment_ObtainCapabilities","resourcedevelopment_StageCapabilities"]+[x[0] for x in BASes_resourcedevelopment_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: lateral-movement

# Remote Services
BASes_lateralmovement_RemoteServices = [
	('Remote Services: Cloud Services', 'lateral-movement', 'T1021.007'),
	('Remote Services: Direct Cloud VM Connections', 'lateral-movement', 'T1021.008'),
	('Remote Services: Distributed Component Object Model', 'lateral-movement', 'T1021.003'),
	('Remote Services: Remote Desktop Protocol', 'lateral-movement', 'T1021.001'),
	('Remote Services: SMB/Windows Admin Shares', 'lateral-movement', 'T1021.002'),
	('Remote Services: SSH', 'lateral-movement', 'T1021.004'),
	('Remote Services: VNC', 'lateral-movement', 'T1021.005'),
	('Remote Services: Windows Remote Management', 'lateral-movement', 'T1021.006'),
]
for i in range(len(BASes_lateralmovement_RemoteServices)):
	if BASes_lateralmovement_RemoteServices[i][-1] in campaign_tactech["lateral-movement"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_lateralmovement_RemoteServices[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_lateralmovement_RemoteServices[i], is_dummy=True)

AT.add_gate_logic("lateralmovement_RemoteServices", "OR", color="#88ccee", children=[ x[0] for x in BASes_lateralmovement_RemoteServices])

# Use Alternate Authentication Material
BASes_lateralmovement_UseAlternateAuthenticationMaterial = [
	('Use Alternate Authentication Material: Application Access Token', 'lateral-movement', 'T1550.001'),
	('Use Alternate Authentication Material: Pass the Hash', 'lateral-movement', 'T1550.002'),
	('Use Alternate Authentication Material: Pass the Ticket', 'lateral-movement', 'T1550.003'),
	('Use Alternate Authentication Material: Web Session Cookie', 'lateral-movement', 'T1550.004'),
]
for i in range(len(BASes_lateralmovement_UseAlternateAuthenticationMaterial)):
	if BASes_lateralmovement_UseAlternateAuthenticationMaterial[i][-1] in campaign_tactech["lateral-movement"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_lateralmovement_UseAlternateAuthenticationMaterial[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_lateralmovement_UseAlternateAuthenticationMaterial[i], is_dummy=True)

AT.add_gate_logic("lateralmovement_UseAlternateAuthenticationMaterial", "OR", color="#88ccee", children=[ x[0] for x in BASes_lateralmovement_UseAlternateAuthenticationMaterial])

# Remote Service Session Hijacking
BASes_lateralmovement_RemoteServiceSessionHijacking = [
	('Remote Service Session Hijacking: RDP Hijacking', 'lateral-movement', 'T1563.002'),
	('Remote Service Session Hijacking: SSH Hijacking', 'lateral-movement', 'T1563.001'),
]
for i in range(len(BASes_lateralmovement_RemoteServiceSessionHijacking)):
	if BASes_lateralmovement_RemoteServiceSessionHijacking[i][-1] in campaign_tactech["lateral-movement"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_lateralmovement_RemoteServiceSessionHijacking[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_lateralmovement_RemoteServiceSessionHijacking[i], is_dummy=True)

AT.add_gate_logic("lateralmovement_RemoteServiceSessionHijacking", "OR", color="#88ccee", children=[ x[0] for x in BASes_lateralmovement_RemoteServiceSessionHijacking])

# direct BASes
BASes_lateralmovement_childless = [
	('Exploitation of Remote Services', 'lateral-movement', 'T1210'),
	('Internal Spearphishing', 'lateral-movement', 'T1534'),
	('Lateral Tool Transfer', 'lateral-movement', 'T1570'),
	('Replication Through Removable Media', 'lateral-movement', 'T1091'),
	('Software Deployment Tools', 'lateral-movement', 'T1072'),
	('Taint Shared Content', 'lateral-movement', 'T1080'),
]
for i in range(len(BASes_lateralmovement_childless)):
	if BASes_lateralmovement_childless[i][-1] in campaign_tactech["lateral-movement"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_lateralmovement_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_lateralmovement_childless[i], is_dummy=True)

AT.add_gate_logic("lateral-movement", "AND", color="#cc79a7", children=["lateralmovement_RemoteServices","lateralmovement_UseAlternateAuthenticationMaterial","lateralmovement_RemoteServiceSessionHijacking"]+[x[0] for x in BASes_lateralmovement_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: credential-access

# Brute Force
BASes_credentialaccess_BruteForce = [
	('Brute Force: Credential Stuffing', 'credential-access', 'T1110.004'),
	('Brute Force: Password Cracking', 'credential-access', 'T1110.002'),
	('Brute Force: Password Guessing', 'credential-access', 'T1110.001'),
	('Brute Force: Password Spraying', 'credential-access', 'T1110.003'),
]
for i in range(len(BASes_credentialaccess_BruteForce)):
	if BASes_credentialaccess_BruteForce[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_BruteForce[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_BruteForce[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_BruteForce", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_BruteForce])

# Forge Web Credentials
BASes_credentialaccess_ForgeWebCredentials = [
	('Forge Web Credentials: SAML Tokens', 'credential-access', 'T1606.002'),
	('Forge Web Credentials: Web Cookies', 'credential-access', 'T1606.001'),
]
for i in range(len(BASes_credentialaccess_ForgeWebCredentials)):
	if BASes_credentialaccess_ForgeWebCredentials[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_ForgeWebCredentials[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_ForgeWebCredentials[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_ForgeWebCredentials", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_ForgeWebCredentials])

# Unsecured Credentials
BASes_credentialaccess_UnsecuredCredentials = [
	('Unsecured Credentials: Bash History', 'credential-access', 'T1552.003'),
	('Unsecured Credentials: Chat Messages', 'credential-access', 'T1552.008'),
	('Unsecured Credentials: Cloud Instance Metadata API', 'credential-access', 'T1552.005'),
	('Unsecured Credentials: Container API', 'credential-access', 'T1552.007'),
	('Unsecured Credentials: Credentials In Files', 'credential-access', 'T1552.001'),
	('Unsecured Credentials: Credentials in Registry', 'credential-access', 'T1552.002'),
	('Unsecured Credentials: Group Policy Preferences', 'credential-access', 'T1552.006'),
	('Unsecured Credentials: Private Keys', 'credential-access', 'T1552.004'),
]
for i in range(len(BASes_credentialaccess_UnsecuredCredentials)):
	if BASes_credentialaccess_UnsecuredCredentials[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_UnsecuredCredentials[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_UnsecuredCredentials[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_UnsecuredCredentials", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_UnsecuredCredentials])

# Adversary-in-the-Middle
BASes_credentialaccess_AdversaryintheMiddle = [
	('Adversary-in-the-Middle: ARP Cache Poisoning', 'credential-access', 'T1557.002'),
	('Adversary-in-the-Middle: DHCP Spoofing', 'credential-access', 'T1557.003'),
	('Adversary-in-the-Middle: LLMNR/NBT-NS Poisoning and SMB Relay', 'credential-access', 'T1557.001'),
]
for i in range(len(BASes_credentialaccess_AdversaryintheMiddle)):
	if BASes_credentialaccess_AdversaryintheMiddle[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_AdversaryintheMiddle[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_AdversaryintheMiddle[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_AdversaryintheMiddle", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_AdversaryintheMiddle])

# Modify Authentication Process
BASes_credentialaccess_ModifyAuthenticationProcess = [
	('Modify Authentication Process: Domain Controller Authentication', 'credential-access', 'T1556.001'),
	('Modify Authentication Process: Hybrid Identity', 'credential-access', 'T1556.007'),
	('Modify Authentication Process: Multi-Factor Authentication', 'credential-access', 'T1556.006'),
	('Modify Authentication Process: Network Device Authentication', 'credential-access', 'T1556.004'),
	('Modify Authentication Process: Network Provider DLL', 'credential-access', 'T1556.008'),
	('Modify Authentication Process: Password Filter DLL', 'credential-access', 'T1556.002'),
	('Modify Authentication Process: Pluggable Authentication Modules', 'credential-access', 'T1556.003'),
	('Modify Authentication Process: Reversible Encryption', 'credential-access', 'T1556.005'),
]
for i in range(len(BASes_credentialaccess_ModifyAuthenticationProcess)):
	if BASes_credentialaccess_ModifyAuthenticationProcess[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_ModifyAuthenticationProcess[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_ModifyAuthenticationProcess[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_ModifyAuthenticationProcess", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_ModifyAuthenticationProcess])

# Steal or Forge Kerberos Tickets
BASes_credentialaccess_StealorForgeKerberosTickets = [
	('Steal or Forge Kerberos Tickets: AS-REP Roasting', 'credential-access', 'T1558.004'),
	('Steal or Forge Kerberos Tickets: Golden Ticket', 'credential-access', 'T1558.001'),
	('Steal or Forge Kerberos Tickets: Kerberoasting', 'credential-access', 'T1558.003'),
	('Steal or Forge Kerberos Tickets: Silver Ticket', 'credential-access', 'T1558.002'),
]
for i in range(len(BASes_credentialaccess_StealorForgeKerberosTickets)):
	if BASes_credentialaccess_StealorForgeKerberosTickets[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_StealorForgeKerberosTickets[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_StealorForgeKerberosTickets[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_StealorForgeKerberosTickets", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_StealorForgeKerberosTickets])

# Input Capture
BASes_credentialaccess_InputCapture = [
	('Input Capture: Credential API Hooking', 'credential-access', 'T1056.004'),
	('Input Capture: GUI Input Capture', 'credential-access', 'T1056.002'),
	('Input Capture: Keylogging', 'credential-access', 'T1056.001'),
	('Input Capture: Web Portal Capture', 'credential-access', 'T1056.003'),
]
for i in range(len(BASes_credentialaccess_InputCapture)):
	if BASes_credentialaccess_InputCapture[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_InputCapture[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_InputCapture[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_InputCapture", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_InputCapture])

# OS Credential Dumping
BASes_credentialaccess_OSCredentialDumping = [
	('OS Credential Dumping: /etc/passwd and /etc/shadow', 'credential-access', 'T1003.008'),
	('OS Credential Dumping: Cached Domain Credentials', 'credential-access', 'T1003.005'),
	('OS Credential Dumping: DCSync', 'credential-access', 'T1003.006'),
	('OS Credential Dumping: LSA Secrets', 'credential-access', 'T1003.004'),
	('OS Credential Dumping: LSASS Memory', 'credential-access', 'T1003.001'),
	('OS Credential Dumping: NTDS', 'credential-access', 'T1003.003'),
	('OS Credential Dumping: Proc Filesystem', 'credential-access', 'T1003.007'),
	('OS Credential Dumping: Security Account Manager', 'credential-access', 'T1003.002'),
]
for i in range(len(BASes_credentialaccess_OSCredentialDumping)):
	if BASes_credentialaccess_OSCredentialDumping[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_OSCredentialDumping[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_OSCredentialDumping[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_OSCredentialDumping", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_OSCredentialDumping])

# Credentials from Password Stores
BASes_credentialaccess_CredentialsfromPasswordStores = [
	('Credentials from Password Stores: Cloud Secrets Management Stores', 'credential-access', 'T1555.006'),
	('Credentials from Password Stores: Credentials from Web Browsers', 'credential-access', 'T1555.003'),
	('Credentials from Password Stores: Keychain', 'credential-access', 'T1555.001'),
	('Credentials from Password Stores: Password Managers', 'credential-access', 'T1555.005'),
	('Credentials from Password Stores: Securityd Memory', 'credential-access', 'T1555.002'),
	('Credentials from Password Stores: Windows Credential Manager', 'credential-access', 'T1555.004'),
]
for i in range(len(BASes_credentialaccess_CredentialsfromPasswordStores)):
	if BASes_credentialaccess_CredentialsfromPasswordStores[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_CredentialsfromPasswordStores[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_CredentialsfromPasswordStores[i], is_dummy=True)

AT.add_gate_logic("credentialaccess_CredentialsfromPasswordStores", "OR", color="#88ccee", children=[ x[0] for x in BASes_credentialaccess_CredentialsfromPasswordStores])

# direct BASes
BASes_credentialaccess_childless = [
	('Exploitation for Credential Access', 'credential-access', 'T1212'),
	('Forced Authentication', 'credential-access', 'T1187'),
	('Multi-Factor Authentication Interception', 'credential-access', 'T1111'),
	('Multi-Factor Authentication Request Generation', 'credential-access', 'T1621'),
	('Network Sniffing', 'credential-access', 'T1040'),
	('Steal Application Access Token', 'credential-access', 'T1528'),
	('Steal Web Session Cookie', 'credential-access', 'T1539'),
	('Steal or Forge Authentication Certificates', 'credential-access', 'T1649'),
]
for i in range(len(BASes_credentialaccess_childless)):
	if BASes_credentialaccess_childless[i][-1] in campaign_tactech["credential-access"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_credentialaccess_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_credentialaccess_childless[i], is_dummy=True)

AT.add_gate_logic("credential-access", "AND", color="#cc79a7", children=["credentialaccess_BruteForce","credentialaccess_ForgeWebCredentials","credentialaccess_UnsecuredCredentials","credentialaccess_AdversaryintheMiddle","credentialaccess_ModifyAuthenticationProcess","credentialaccess_StealorForgeKerberosTickets","credentialaccess_InputCapture","credentialaccess_OSCredentialDumping","credentialaccess_CredentialsfromPasswordStores"]+[x[0] for x in BASes_credentialaccess_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: impact

# Network Denial of Service
BASes_impact_NetworkDenialofService = [
	('Network Denial of Service: Direct Network Flood', 'impact', 'T1498.001'),
	('Network Denial of Service: Reflection Amplification', 'impact', 'T1498.002'),
]
for i in range(len(BASes_impact_NetworkDenialofService)):
	if BASes_impact_NetworkDenialofService[i][-1] in campaign_tactech["impact"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_impact_NetworkDenialofService[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_impact_NetworkDenialofService[i], is_dummy=True)

AT.add_gate_logic("impact_NetworkDenialofService", "OR", color="#88ccee", children=[ x[0] for x in BASes_impact_NetworkDenialofService])

# Endpoint Denial of Service
BASes_impact_EndpointDenialofService = [
	('Endpoint Denial of Service: Application Exhaustion Flood', 'impact', 'T1499.003'),
	('Endpoint Denial of Service: Application or System Exploitation', 'impact', 'T1499.004'),
	('Endpoint Denial of Service: OS Exhaustion Flood', 'impact', 'T1499.001'),
	('Endpoint Denial of Service: Service Exhaustion Flood', 'impact', 'T1499.002'),
]
for i in range(len(BASes_impact_EndpointDenialofService)):
	if BASes_impact_EndpointDenialofService[i][-1] in campaign_tactech["impact"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_impact_EndpointDenialofService[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_impact_EndpointDenialofService[i], is_dummy=True)

AT.add_gate_logic("impact_EndpointDenialofService", "OR", color="#88ccee", children=[ x[0] for x in BASes_impact_EndpointDenialofService])

# Disk Wipe
BASes_impact_DiskWipe = [
	('Disk Wipe: Disk Content Wipe', 'impact', 'T1561.001'),
	('Disk Wipe: Disk Structure Wipe', 'impact', 'T1561.002'),
]
for i in range(len(BASes_impact_DiskWipe)):
	if BASes_impact_DiskWipe[i][-1] in campaign_tactech["impact"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_impact_DiskWipe[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_impact_DiskWipe[i], is_dummy=True)

AT.add_gate_logic("impact_DiskWipe", "OR", color="#88ccee", children=[ x[0] for x in BASes_impact_DiskWipe])

# Defacement
BASes_impact_Defacement = [
	('Defacement: External Defacement', 'impact', 'T1491.002'),
	('Defacement: Internal Defacement', 'impact', 'T1491.001'),
]
for i in range(len(BASes_impact_Defacement)):
	if BASes_impact_Defacement[i][-1] in campaign_tactech["impact"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_impact_Defacement[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_impact_Defacement[i], is_dummy=True)

AT.add_gate_logic("impact_Defacement", "OR", color="#88ccee", children=[ x[0] for x in BASes_impact_Defacement])

# Data Manipulation
BASes_impact_DataManipulation = [
	('Data Manipulation: Runtime Data Manipulation', 'impact', 'T1565.003'),
	('Data Manipulation: Stored Data Manipulation', 'impact', 'T1565.001'),
	('Data Manipulation: Transmitted Data Manipulation', 'impact', 'T1565.002'),
]
for i in range(len(BASes_impact_DataManipulation)):
	if BASes_impact_DataManipulation[i][-1] in campaign_tactech["impact"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_impact_DataManipulation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_impact_DataManipulation[i], is_dummy=True)

AT.add_gate_logic("impact_DataManipulation", "OR", color="#88ccee", children=[ x[0] for x in BASes_impact_DataManipulation])

# direct BASes
BASes_impact_childless = [
	('Account Access Removal', 'impact', 'T1531'),
	('Data Destruction', 'impact', 'T1485'),
	('Data Encrypted for Impact', 'impact', 'T1486'),
	('Financial Theft', 'impact', 'T1657'),
	('Firmware Corruption', 'impact', 'T1495'),
	('Inhibit System Recovery', 'impact', 'T1490'),
	('Resource Hijacking', 'impact', 'T1496'),
	('Service Stop', 'impact', 'T1489'),
	('System Shutdown/Reboot', 'impact', 'T1529'),
]
for i in range(len(BASes_impact_childless)):
	if BASes_impact_childless[i][-1] in campaign_tactech["impact"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_impact_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_impact_childless[i], is_dummy=True)

AT.add_gate_logic("impact", "AND", color="#cc79a7", children=["impact_NetworkDenialofService","impact_EndpointDenialofService","impact_DiskWipe","impact_Defacement","impact_DataManipulation"]+[x[0] for x in BASes_impact_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: exfiltration

# Exfiltration Over Alternative Protocol
BASes_exfiltration_ExfiltrationOverAlternativeProtocol = [
	('Exfiltration Over Alternative Protocol: Exfiltration Over Asymmetric Encrypted Non-C2 Protocol', 'exfiltration', 'T1048.002'),
	('Exfiltration Over Alternative Protocol: Exfiltration Over Symmetric Encrypted Non-C2 Protocol', 'exfiltration', 'T1048.001'),
	('Exfiltration Over Alternative Protocol: Exfiltration Over Unencrypted Non-C2 Protocol', 'exfiltration', 'T1048.003'),
]
for i in range(len(BASes_exfiltration_ExfiltrationOverAlternativeProtocol)):
	if BASes_exfiltration_ExfiltrationOverAlternativeProtocol[i][-1] in campaign_tactech["exfiltration"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_exfiltration_ExfiltrationOverAlternativeProtocol[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_exfiltration_ExfiltrationOverAlternativeProtocol[i], is_dummy=True)

AT.add_gate_logic("exfiltration_ExfiltrationOverAlternativeProtocol", "OR", color="#88ccee", children=[ x[0] for x in BASes_exfiltration_ExfiltrationOverAlternativeProtocol])

# Exfiltration Over Web Service
BASes_exfiltration_ExfiltrationOverWebService = [
	('Exfiltration Over Web Service: Exfiltration Over Webhook', 'exfiltration', 'T1567.004'),
	('Exfiltration Over Web Service: Exfiltration to Cloud Storage', 'exfiltration', 'T1567.002'),
	('Exfiltration Over Web Service: Exfiltration to Code Repository', 'exfiltration', 'T1567.001'),
	('Exfiltration Over Web Service: Exfiltration to Text Storage Sites', 'exfiltration', 'T1567.003'),
]
for i in range(len(BASes_exfiltration_ExfiltrationOverWebService)):
	if BASes_exfiltration_ExfiltrationOverWebService[i][-1] in campaign_tactech["exfiltration"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_exfiltration_ExfiltrationOverWebService[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_exfiltration_ExfiltrationOverWebService[i], is_dummy=True)

AT.add_gate_logic("exfiltration_ExfiltrationOverWebService", "OR", color="#88ccee", children=[ x[0] for x in BASes_exfiltration_ExfiltrationOverWebService])

# direct BASes
BASes_exfiltration_childless = [
	('Automated Exfiltration: Traffic Duplication', 'exfiltration', 'T1020.001'),
	('Data Transfer Size Limits', 'exfiltration', 'T1030'),
	('Exfiltration Over C2 Channel', 'exfiltration', 'T1041'),
	('Exfiltration Over Other Network Medium: Exfiltration Over Bluetooth', 'exfiltration', 'T1011.001'),
	('Exfiltration Over Physical Medium: Exfiltration over USB', 'exfiltration', 'T1052.001'),
	('Scheduled Transfer', 'exfiltration', 'T1029'),
	('Transfer Data to Cloud Account', 'exfiltration', 'T1537'),
]
for i in range(len(BASes_exfiltration_childless)):
	if BASes_exfiltration_childless[i][-1] in campaign_tactech["exfiltration"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_exfiltration_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_exfiltration_childless[i], is_dummy=True)

AT.add_gate_logic("exfiltration", "AND", color="#cc79a7", children=["exfiltration_ExfiltrationOverAlternativeProtocol","exfiltration_ExfiltrationOverWebService"]+[x[0] for x in BASes_exfiltration_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: defense-evasion

# Abuse Elevation Control Mechanism
BASes_defenseevasion_AbuseElevationControlMechanism = [
	('Abuse Elevation Control Mechanism: Bypass User Account Control', 'defense-evasion', 'T1548.002'),
	('Abuse Elevation Control Mechanism: Elevated Execution with Prompt', 'defense-evasion', 'T1548.004'),
	('Abuse Elevation Control Mechanism: Setuid and Setgid', 'defense-evasion', 'T1548.001'),
	('Abuse Elevation Control Mechanism: Sudo and Sudo Caching', 'defense-evasion', 'T1548.003'),
	('Abuse Elevation Control Mechanism: Temporary Elevated Cloud Access', 'defense-evasion', 'T1548.005'),
]
for i in range(len(BASes_defenseevasion_AbuseElevationControlMechanism)):
	if BASes_defenseevasion_AbuseElevationControlMechanism[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_AbuseElevationControlMechanism[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_AbuseElevationControlMechanism[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_AbuseElevationControlMechanism", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_AbuseElevationControlMechanism])

# Modify Cloud Compute Infrastructure
BASes_defenseevasion_ModifyCloudComputeInfrastructure = [
	('Modify Cloud Compute Infrastructure: Create Cloud Instance', 'defense-evasion', 'T1578.002'),
	('Modify Cloud Compute Infrastructure: Create Snapshot', 'defense-evasion', 'T1578.001'),
	('Modify Cloud Compute Infrastructure: Delete Cloud Instance', 'defense-evasion', 'T1578.003'),
	('Modify Cloud Compute Infrastructure: Modify Cloud Compute Configurations', 'defense-evasion', 'T1578.005'),
	('Modify Cloud Compute Infrastructure: Revert Cloud Instance', 'defense-evasion', 'T1578.004'),
]
for i in range(len(BASes_defenseevasion_ModifyCloudComputeInfrastructure)):
	if BASes_defenseevasion_ModifyCloudComputeInfrastructure[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ModifyCloudComputeInfrastructure[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ModifyCloudComputeInfrastructure[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ModifyCloudComputeInfrastructure", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ModifyCloudComputeInfrastructure])

# Domain Policy Modification
BASes_defenseevasion_DomainPolicyModification = [
	('Domain Policy Modification: Domain Trust Modification', 'defense-evasion', 'T1484.002'),
	('Domain Policy Modification: Group Policy Modification', 'defense-evasion', 'T1484.001'),
]
for i in range(len(BASes_defenseevasion_DomainPolicyModification)):
	if BASes_defenseevasion_DomainPolicyModification[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_DomainPolicyModification[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_DomainPolicyModification[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_DomainPolicyModification", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_DomainPolicyModification])

# Obfuscated Files or Information
BASes_defenseevasion_ObfuscatedFilesorInformation = [
	('Obfuscated Files or Information: Binary Padding', 'defense-evasion', 'T1027.001'),
	('Obfuscated Files or Information: Command Obfuscation', 'defense-evasion', 'T1027.010'),
	('Obfuscated Files or Information: Compile After Delivery', 'defense-evasion', 'T1027.004'),
	('Obfuscated Files or Information: Dynamic API Resolution', 'defense-evasion', 'T1027.007'),
	('Obfuscated Files or Information: Embedded Payloads', 'defense-evasion', 'T1027.009'),
	('Obfuscated Files or Information: Fileless Storage', 'defense-evasion', 'T1027.011'),
	('Obfuscated Files or Information: HTML Smuggling', 'defense-evasion', 'T1027.006'),
	('Obfuscated Files or Information: Indicator Removal from Tools', 'defense-evasion', 'T1027.005'),
	('Obfuscated Files or Information: LNK Icon Smuggling', 'defense-evasion', 'T1027.012'),
	('Obfuscated Files or Information: Software Packing', 'defense-evasion', 'T1027.002'),
	('Obfuscated Files or Information: Steganography', 'defense-evasion', 'T1027.003'),
	('Obfuscated Files or Information: Stripped Payloads', 'defense-evasion', 'T1027.008'),
]
for i in range(len(BASes_defenseevasion_ObfuscatedFilesorInformation)):
	if BASes_defenseevasion_ObfuscatedFilesorInformation[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ObfuscatedFilesorInformation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ObfuscatedFilesorInformation[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ObfuscatedFilesorInformation", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ObfuscatedFilesorInformation])

# Use Alternate Authentication Material
BASes_defenseevasion_UseAlternateAuthenticationMaterial = [
	('Use Alternate Authentication Material: Application Access Token', 'defense-evasion', 'T1550.001'),
	('Use Alternate Authentication Material: Pass the Hash', 'defense-evasion', 'T1550.002'),
	('Use Alternate Authentication Material: Pass the Ticket', 'defense-evasion', 'T1550.003'),
	('Use Alternate Authentication Material: Web Session Cookie', 'defense-evasion', 'T1550.004'),
]
for i in range(len(BASes_defenseevasion_UseAlternateAuthenticationMaterial)):
	if BASes_defenseevasion_UseAlternateAuthenticationMaterial[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_UseAlternateAuthenticationMaterial[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_UseAlternateAuthenticationMaterial[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_UseAlternateAuthenticationMaterial", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_UseAlternateAuthenticationMaterial])

# Subvert Trust Controls
BASes_defenseevasion_SubvertTrustControls = [
	('Subvert Trust Controls: Code Signing', 'defense-evasion', 'T1553.002'),
	('Subvert Trust Controls: Code Signing Policy Modification', 'defense-evasion', 'T1553.006'),
	('Subvert Trust Controls: Gatekeeper Bypass', 'defense-evasion', 'T1553.001'),
	('Subvert Trust Controls: Install Root Certificate', 'defense-evasion', 'T1553.004'),
	('Subvert Trust Controls: Mark-of-the-Web Bypass', 'defense-evasion', 'T1553.005'),
	('Subvert Trust Controls: SIP and Trust Provider Hijacking', 'defense-evasion', 'T1553.003'),
]
for i in range(len(BASes_defenseevasion_SubvertTrustControls)):
	if BASes_defenseevasion_SubvertTrustControls[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_SubvertTrustControls[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_SubvertTrustControls[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_SubvertTrustControls", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_SubvertTrustControls])

# System Binary Proxy Execution
BASes_defenseevasion_SystemBinaryProxyExecution = [
	('System Binary Proxy Execution: CMSTP', 'defense-evasion', 'T1218.003'),
	('System Binary Proxy Execution: Compiled HTML File', 'defense-evasion', 'T1218.001'),
	('System Binary Proxy Execution: Control Panel', 'defense-evasion', 'T1218.002'),
	('System Binary Proxy Execution: InstallUtil', 'defense-evasion', 'T1218.004'),
	('System Binary Proxy Execution: MMC', 'defense-evasion', 'T1218.014'),
	('System Binary Proxy Execution: Mavinject', 'defense-evasion', 'T1218.013'),
	('System Binary Proxy Execution: Mshta', 'defense-evasion', 'T1218.005'),
	('System Binary Proxy Execution: Msiexec', 'defense-evasion', 'T1218.007'),
	('System Binary Proxy Execution: Odbcconf', 'defense-evasion', 'T1218.008'),
	('System Binary Proxy Execution: Regsvcs/Regasm', 'defense-evasion', 'T1218.009'),
	('System Binary Proxy Execution: Regsvr32', 'defense-evasion', 'T1218.010'),
	('System Binary Proxy Execution: Rundll32', 'defense-evasion', 'T1218.011'),
	('System Binary Proxy Execution: Verclsid', 'defense-evasion', 'T1218.012'),
]
for i in range(len(BASes_defenseevasion_SystemBinaryProxyExecution)):
	if BASes_defenseevasion_SystemBinaryProxyExecution[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_SystemBinaryProxyExecution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_SystemBinaryProxyExecution[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_SystemBinaryProxyExecution", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_SystemBinaryProxyExecution])

# Weaken Encryption
BASes_defenseevasion_WeakenEncryption = [
	('Weaken Encryption: Disable Crypto Hardware', 'defense-evasion', 'T1600.002'),
	('Weaken Encryption: Reduce Key Space', 'defense-evasion', 'T1600.001'),
]
for i in range(len(BASes_defenseevasion_WeakenEncryption)):
	if BASes_defenseevasion_WeakenEncryption[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_WeakenEncryption[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_WeakenEncryption[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_WeakenEncryption", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_WeakenEncryption])

# Pre-OS Boot
BASes_defenseevasion_PreOSBoot = [
	('Pre-OS Boot: Bootkit', 'defense-evasion', 'T1542.003'),
	('Pre-OS Boot: Component Firmware', 'defense-evasion', 'T1542.002'),
	('Pre-OS Boot: ROMMONkit', 'defense-evasion', 'T1542.004'),
	('Pre-OS Boot: System Firmware', 'defense-evasion', 'T1542.001'),
	('Pre-OS Boot: TFTP Boot', 'defense-evasion', 'T1542.005'),
]
for i in range(len(BASes_defenseevasion_PreOSBoot)):
	if BASes_defenseevasion_PreOSBoot[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_PreOSBoot[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_PreOSBoot[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_PreOSBoot", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_PreOSBoot])

# Virtualization/Sandbox Evasion
BASes_defenseevasion_VirtualizationSandboxEvasion = [
	('Virtualization/Sandbox Evasion: System Checks', 'defense-evasion', 'T1497.001'),
	('Virtualization/Sandbox Evasion: Time Based Evasion', 'defense-evasion', 'T1497.003'),
	('Virtualization/Sandbox Evasion: User Activity Based Checks', 'defense-evasion', 'T1497.002'),
]
for i in range(len(BASes_defenseevasion_VirtualizationSandboxEvasion)):
	if BASes_defenseevasion_VirtualizationSandboxEvasion[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_VirtualizationSandboxEvasion[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_VirtualizationSandboxEvasion[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_VirtualizationSandboxEvasion", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_VirtualizationSandboxEvasion])

# Valid Accounts
BASes_defenseevasion_ValidAccounts = [
	('Valid Accounts: Cloud Accounts', 'defense-evasion', 'T1078.004'),
	('Valid Accounts: Default Accounts', 'defense-evasion', 'T1078.001'),
	('Valid Accounts: Domain Accounts', 'defense-evasion', 'T1078.002'),
	('Valid Accounts: Local Accounts', 'defense-evasion', 'T1078.003'),
]
for i in range(len(BASes_defenseevasion_ValidAccounts)):
	if BASes_defenseevasion_ValidAccounts[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ValidAccounts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ValidAccounts[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ValidAccounts", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ValidAccounts])

# Hide Artifacts
BASes_defenseevasion_HideArtifacts = [
	('Hide Artifacts: Email Hiding Rules', 'defense-evasion', 'T1564.008'),
	('Hide Artifacts: Hidden File System', 'defense-evasion', 'T1564.005'),
	('Hide Artifacts: Hidden Files and Directories', 'defense-evasion', 'T1564.001'),
	('Hide Artifacts: Hidden Users', 'defense-evasion', 'T1564.002'),
	('Hide Artifacts: Hidden Window', 'defense-evasion', 'T1564.003'),
	('Hide Artifacts: Ignore Process Interrupts', 'defense-evasion', 'T1564.011'),
	('Hide Artifacts: NTFS File Attributes', 'defense-evasion', 'T1564.004'),
	('Hide Artifacts: Process Argument Spoofing', 'defense-evasion', 'T1564.010'),
	('Hide Artifacts: Resource Forking', 'defense-evasion', 'T1564.009'),
	('Hide Artifacts: Run Virtual Instance', 'defense-evasion', 'T1564.006'),
	('Hide Artifacts: VBA Stomping', 'defense-evasion', 'T1564.007'),
]
for i in range(len(BASes_defenseevasion_HideArtifacts)):
	if BASes_defenseevasion_HideArtifacts[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_HideArtifacts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_HideArtifacts[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_HideArtifacts", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_HideArtifacts])

# Masquerading
BASes_defenseevasion_Masquerading = [
	('Masquerading: Break Process Trees', 'defense-evasion', 'T1036.009'),
	('Masquerading: Double File Extension', 'defense-evasion', 'T1036.007'),
	('Masquerading: Invalid Code Signature', 'defense-evasion', 'T1036.001'),
	('Masquerading: Masquerade File Type', 'defense-evasion', 'T1036.008'),
	('Masquerading: Masquerade Task or Service', 'defense-evasion', 'T1036.004'),
	('Masquerading: Match Legitimate Name or Location', 'defense-evasion', 'T1036.005'),
	('Masquerading: Rename System Utilities', 'defense-evasion', 'T1036.003'),
	('Masquerading: Right-to-Left Override', 'defense-evasion', 'T1036.002'),
	('Masquerading: Space after Filename', 'defense-evasion', 'T1036.006'),
]
for i in range(len(BASes_defenseevasion_Masquerading)):
	if BASes_defenseevasion_Masquerading[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_Masquerading[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_Masquerading[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_Masquerading", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_Masquerading])

# Modify Authentication Process
BASes_defenseevasion_ModifyAuthenticationProcess = [
	('Modify Authentication Process: Domain Controller Authentication', 'defense-evasion', 'T1556.001'),
	('Modify Authentication Process: Hybrid Identity', 'defense-evasion', 'T1556.007'),
	('Modify Authentication Process: Multi-Factor Authentication', 'defense-evasion', 'T1556.006'),
	('Modify Authentication Process: Network Device Authentication', 'defense-evasion', 'T1556.004'),
	('Modify Authentication Process: Network Provider DLL', 'defense-evasion', 'T1556.008'),
	('Modify Authentication Process: Password Filter DLL', 'defense-evasion', 'T1556.002'),
	('Modify Authentication Process: Pluggable Authentication Modules', 'defense-evasion', 'T1556.003'),
	('Modify Authentication Process: Reversible Encryption', 'defense-evasion', 'T1556.005'),
]
for i in range(len(BASes_defenseevasion_ModifyAuthenticationProcess)):
	if BASes_defenseevasion_ModifyAuthenticationProcess[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ModifyAuthenticationProcess[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ModifyAuthenticationProcess[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ModifyAuthenticationProcess", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ModifyAuthenticationProcess])

# File and Directory Permissions Modification
BASes_defenseevasion_FileandDirectoryPermissionsModification = [
	('File and Directory Permissions Modification: Linux and Mac File and Directory Permissions Modification', 'defense-evasion', 'T1222.002'),
	('File and Directory Permissions Modification: Windows File and Directory Permissions Modification', 'defense-evasion', 'T1222.001'),
]
for i in range(len(BASes_defenseevasion_FileandDirectoryPermissionsModification)):
	if BASes_defenseevasion_FileandDirectoryPermissionsModification[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_FileandDirectoryPermissionsModification[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_FileandDirectoryPermissionsModification[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_FileandDirectoryPermissionsModification", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_FileandDirectoryPermissionsModification])

# Impair Defenses
BASes_defenseevasion_ImpairDefenses = [
	('Impair Defenses: Disable Windows Event Logging', 'defense-evasion', 'T1562.002'),
	('Impair Defenses: Disable or Modify Cloud Firewall', 'defense-evasion', 'T1562.007'),
	('Impair Defenses: Disable or Modify Cloud Logs', 'defense-evasion', 'T1562.008'),
	('Impair Defenses: Disable or Modify Linux Audit System', 'defense-evasion', 'T1562.012'),
	('Impair Defenses: Disable or Modify System Firewall', 'defense-evasion', 'T1562.004'),
	('Impair Defenses: Disable or Modify Tools', 'defense-evasion', 'T1562.001'),
	('Impair Defenses: Downgrade Attack', 'defense-evasion', 'T1562.010'),
	('Impair Defenses: Impair Command History Logging', 'defense-evasion', 'T1562.003'),
	('Impair Defenses: Indicator Blocking', 'defense-evasion', 'T1562.006'),
	('Impair Defenses: Safe Mode Boot', 'defense-evasion', 'T1562.009'),
	('Impair Defenses: Spoof Security Alerting', 'defense-evasion', 'T1562.011'),
]
for i in range(len(BASes_defenseevasion_ImpairDefenses)):
	if BASes_defenseevasion_ImpairDefenses[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ImpairDefenses[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ImpairDefenses[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ImpairDefenses", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ImpairDefenses])

# Hijack Execution Flow
BASes_defenseevasion_HijackExecutionFlow = [
	('Hijack Execution Flow: COR_PROFILER', 'defense-evasion', 'T1574.012'),
	('Hijack Execution Flow: DLL Search Order Hijacking', 'defense-evasion', 'T1574.001'),
	('Hijack Execution Flow: DLL Side-Loading', 'defense-evasion', 'T1574.002'),
	('Hijack Execution Flow: Dylib Hijacking', 'defense-evasion', 'T1574.004'),
	('Hijack Execution Flow: Dynamic Linker Hijacking', 'defense-evasion', 'T1574.006'),
	('Hijack Execution Flow: Executable Installer File Permissions Weakness', 'defense-evasion', 'T1574.005'),
	('Hijack Execution Flow: KernelCallbackTable', 'defense-evasion', 'T1574.013'),
	('Hijack Execution Flow: Path Interception by PATH Environment Variable', 'defense-evasion', 'T1574.007'),
	('Hijack Execution Flow: Path Interception by Search Order Hijacking', 'defense-evasion', 'T1574.008'),
	('Hijack Execution Flow: Path Interception by Unquoted Path', 'defense-evasion', 'T1574.009'),
	('Hijack Execution Flow: Services File Permissions Weakness', 'defense-evasion', 'T1574.010'),
	('Hijack Execution Flow: Services Registry Permissions Weakness', 'defense-evasion', 'T1574.011'),
]
for i in range(len(BASes_defenseevasion_HijackExecutionFlow)):
	if BASes_defenseevasion_HijackExecutionFlow[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_HijackExecutionFlow[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_HijackExecutionFlow[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_HijackExecutionFlow", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_HijackExecutionFlow])

# Modify System Image
BASes_defenseevasion_ModifySystemImage = [
	('Modify System Image: Downgrade System Image', 'defense-evasion', 'T1601.002'),
	('Modify System Image: Patch System Image', 'defense-evasion', 'T1601.001'),
]
for i in range(len(BASes_defenseevasion_ModifySystemImage)):
	if BASes_defenseevasion_ModifySystemImage[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ModifySystemImage[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ModifySystemImage[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ModifySystemImage", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ModifySystemImage])

# Traffic Signaling
BASes_defenseevasion_TrafficSignaling = [
	('Traffic Signaling: Port Knocking', 'defense-evasion', 'T1205.001'),
	('Traffic Signaling: Socket Filters', 'defense-evasion', 'T1205.002'),
]
for i in range(len(BASes_defenseevasion_TrafficSignaling)):
	if BASes_defenseevasion_TrafficSignaling[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_TrafficSignaling[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_TrafficSignaling[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_TrafficSignaling", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_TrafficSignaling])

# Access Token Manipulation
BASes_defenseevasion_AccessTokenManipulation = [
	('Access Token Manipulation: Create Process with Token', 'defense-evasion', 'T1134.002'),
	('Access Token Manipulation: Make and Impersonate Token', 'defense-evasion', 'T1134.003'),
	('Access Token Manipulation: Parent PID Spoofing', 'defense-evasion', 'T1134.004'),
	('Access Token Manipulation: SID-History Injection', 'defense-evasion', 'T1134.005'),
	('Access Token Manipulation: Token Impersonation/Theft', 'defense-evasion', 'T1134.001'),
]
for i in range(len(BASes_defenseevasion_AccessTokenManipulation)):
	if BASes_defenseevasion_AccessTokenManipulation[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_AccessTokenManipulation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_AccessTokenManipulation[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_AccessTokenManipulation", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_AccessTokenManipulation])

# Process Injection
BASes_defenseevasion_ProcessInjection = [
	('Process Injection: Asynchronous Procedure Call', 'defense-evasion', 'T1055.004'),
	('Process Injection: Dynamic-link Library Injection', 'defense-evasion', 'T1055.001'),
	('Process Injection: Extra Window Memory Injection', 'defense-evasion', 'T1055.011'),
	('Process Injection: ListPlanting', 'defense-evasion', 'T1055.015'),
	('Process Injection: Portable Executable Injection', 'defense-evasion', 'T1055.002'),
	('Process Injection: Proc Memory', 'defense-evasion', 'T1055.009'),
	('Process Injection: Process Doppelgänging', 'defense-evasion', 'T1055.013'),
	('Process Injection: Process Hollowing', 'defense-evasion', 'T1055.012'),
	('Process Injection: Ptrace System Calls', 'defense-evasion', 'T1055.008'),
	('Process Injection: Thread Execution Hijacking', 'defense-evasion', 'T1055.003'),
	('Process Injection: Thread Local Storage', 'defense-evasion', 'T1055.005'),
	('Process Injection: VDSO Hijacking', 'defense-evasion', 'T1055.014'),
]
for i in range(len(BASes_defenseevasion_ProcessInjection)):
	if BASes_defenseevasion_ProcessInjection[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_ProcessInjection[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_ProcessInjection[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_ProcessInjection", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_ProcessInjection])

# Indicator Removal
BASes_defenseevasion_IndicatorRemoval = [
	('Indicator Removal: Clear Command History', 'defense-evasion', 'T1070.003'),
	('Indicator Removal: Clear Linux or Mac System Logs', 'defense-evasion', 'T1070.002'),
	('Indicator Removal: Clear Mailbox Data', 'defense-evasion', 'T1070.008'),
	('Indicator Removal: Clear Network Connection History and Configurations', 'defense-evasion', 'T1070.007'),
	('Indicator Removal: Clear Persistence', 'defense-evasion', 'T1070.009'),
	('Indicator Removal: Clear Windows Event Logs', 'defense-evasion', 'T1070.001'),
	('Indicator Removal: File Deletion', 'defense-evasion', 'T1070.004'),
	('Indicator Removal: Network Share Connection Removal', 'defense-evasion', 'T1070.005'),
	('Indicator Removal: Timestomp', 'defense-evasion', 'T1070.006'),
]
for i in range(len(BASes_defenseevasion_IndicatorRemoval)):
	if BASes_defenseevasion_IndicatorRemoval[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_IndicatorRemoval[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_IndicatorRemoval[i], is_dummy=True)

AT.add_gate_logic("defenseevasion_IndicatorRemoval", "OR", color="#88ccee", children=[ x[0] for x in BASes_defenseevasion_IndicatorRemoval])

# direct BASes
BASes_defenseevasion_childless = [
	('BITS Jobs', 'defense-evasion', 'T1197'),
	('Build Image on Host', 'defense-evasion', 'T1612'),
	('Debugger Evasion', 'defense-evasion', 'T1622'),
	('Deobfuscate/Decode Files or Information', 'defense-evasion', 'T1140'),
	('Deploy Container', 'defense-evasion', 'T1610'),
	('Direct Volume Access', 'defense-evasion', 'T1006'),
	('Execution Guardrails: Environmental Keying', 'defense-evasion', 'T1480.001'),
	('Exploitation for Defense Evasion', 'defense-evasion', 'T1211'),
	('Impersonation', 'defense-evasion', 'T1656'),
	('Indirect Command Execution', 'defense-evasion', 'T1202'),
	('Modify Registry', 'defense-evasion', 'T1112'),
	('Network Boundary Bridging: Network Address Translation Traversal', 'defense-evasion', 'T1599.001'),
	('Plist File Modification', 'defense-evasion', 'T1647'),
	('Reflective Code Loading', 'defense-evasion', 'T1620'),
	('Rogue Domain Controller', 'defense-evasion', 'T1207'),
	('Rootkit', 'defense-evasion', 'T1014'),
	('System Script Proxy Execution: PubPrn', 'defense-evasion', 'T1216.001'),
	('Template Injection', 'defense-evasion', 'T1221'),
	('Trusted Developer Utilities Proxy Execution: MSBuild', 'defense-evasion', 'T1127.001'),
	('Unused/Unsupported Cloud Regions', 'defense-evasion', 'T1535'),
	('XSL Script Processing', 'defense-evasion', 'T1220'),
]
for i in range(len(BASes_defenseevasion_childless)):
	if BASes_defenseevasion_childless[i][-1] in campaign_tactech["defense-evasion"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_defenseevasion_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_defenseevasion_childless[i], is_dummy=True)

AT.add_gate_logic("defense-evasion", "AND", color="#cc79a7", children=["defenseevasion_AbuseElevationControlMechanism","defenseevasion_ModifyCloudComputeInfrastructure","defenseevasion_DomainPolicyModification","defenseevasion_ObfuscatedFilesorInformation","defenseevasion_UseAlternateAuthenticationMaterial","defenseevasion_SubvertTrustControls","defenseevasion_SystemBinaryProxyExecution","defenseevasion_WeakenEncryption","defenseevasion_PreOSBoot","defenseevasion_VirtualizationSandboxEvasion","defenseevasion_ValidAccounts","defenseevasion_HideArtifacts","defenseevasion_Masquerading","defenseevasion_ModifyAuthenticationProcess","defenseevasion_FileandDirectoryPermissionsModification","defenseevasion_ImpairDefenses","defenseevasion_HijackExecutionFlow","defenseevasion_ModifySystemImage","defenseevasion_TrafficSignaling","defenseevasion_AccessTokenManipulation","defenseevasion_ProcessInjection","defenseevasion_IndicatorRemoval"]+[x[0] for x in BASes_defenseevasion_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: execution

# System Services
BASes_execution_SystemServices = [
	('System Services: Launchctl', 'execution', 'T1569.001'),
	('System Services: Service Execution', 'execution', 'T1569.002'),
]
for i in range(len(BASes_execution_SystemServices)):
	if BASes_execution_SystemServices[i][-1] in campaign_tactech["execution"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_execution_SystemServices[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_execution_SystemServices[i], is_dummy=True)

AT.add_gate_logic("execution_SystemServices", "OR", color="#88ccee", children=[ x[0] for x in BASes_execution_SystemServices])

# Scheduled Task/Job
BASes_execution_ScheduledTaskJob = [
	('Scheduled Task/Job: At', 'execution', 'T1053.002'),
	('Scheduled Task/Job: Container Orchestration Job', 'execution', 'T1053.007'),
	('Scheduled Task/Job: Cron', 'execution', 'T1053.003'),
	('Scheduled Task/Job: Scheduled Task', 'execution', 'T1053.005'),
	('Scheduled Task/Job: Systemd Timers', 'execution', 'T1053.006'),
]
for i in range(len(BASes_execution_ScheduledTaskJob)):
	if BASes_execution_ScheduledTaskJob[i][-1] in campaign_tactech["execution"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_execution_ScheduledTaskJob[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_execution_ScheduledTaskJob[i], is_dummy=True)

AT.add_gate_logic("execution_ScheduledTaskJob", "OR", color="#88ccee", children=[ x[0] for x in BASes_execution_ScheduledTaskJob])

# Command and Scripting Interpreter
BASes_execution_CommandandScriptingInterpreter = [
	('Command and Scripting Interpreter: AppleScript', 'execution', 'T1059.002'),
	('Command and Scripting Interpreter: Cloud API', 'execution', 'T1059.009'),
	('Command and Scripting Interpreter: JavaScript', 'execution', 'T1059.007'),
	('Command and Scripting Interpreter: Network Device CLI', 'execution', 'T1059.008'),
	('Command and Scripting Interpreter: PowerShell', 'execution', 'T1059.001'),
	('Command and Scripting Interpreter: Python', 'execution', 'T1059.006'),
	('Command and Scripting Interpreter: Unix Shell', 'execution', 'T1059.004'),
	('Command and Scripting Interpreter: Visual Basic', 'execution', 'T1059.005'),
	('Command and Scripting Interpreter: Windows Command Shell', 'execution', 'T1059.003'),
]
for i in range(len(BASes_execution_CommandandScriptingInterpreter)):
	if BASes_execution_CommandandScriptingInterpreter[i][-1] in campaign_tactech["execution"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_execution_CommandandScriptingInterpreter[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_execution_CommandandScriptingInterpreter[i], is_dummy=True)

AT.add_gate_logic("execution_CommandandScriptingInterpreter", "OR", color="#88ccee", children=[ x[0] for x in BASes_execution_CommandandScriptingInterpreter])

# User Execution
BASes_execution_UserExecution = [
	('User Execution: Malicious File', 'execution', 'T1204.002'),
	('User Execution: Malicious Image', 'execution', 'T1204.003'),
	('User Execution: Malicious Link', 'execution', 'T1204.001'),
]
for i in range(len(BASes_execution_UserExecution)):
	if BASes_execution_UserExecution[i][-1] in campaign_tactech["execution"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_execution_UserExecution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_execution_UserExecution[i], is_dummy=True)

AT.add_gate_logic("execution_UserExecution", "OR", color="#88ccee", children=[ x[0] for x in BASes_execution_UserExecution])

# Inter-Process Communication
BASes_execution_InterProcessCommunication = [
	('Inter-Process Communication: Component Object Model', 'execution', 'T1559.001'),
	('Inter-Process Communication: Dynamic Data Exchange', 'execution', 'T1559.002'),
	('Inter-Process Communication: XPC Services', 'execution', 'T1559.003'),
]
for i in range(len(BASes_execution_InterProcessCommunication)):
	if BASes_execution_InterProcessCommunication[i][-1] in campaign_tactech["execution"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_execution_InterProcessCommunication[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_execution_InterProcessCommunication[i], is_dummy=True)

AT.add_gate_logic("execution_InterProcessCommunication", "OR", color="#88ccee", children=[ x[0] for x in BASes_execution_InterProcessCommunication])

# direct BASes
BASes_execution_childless = [
	('Cloud Administration Command', 'execution', 'T1651'),
	('Container Administration Command', 'execution', 'T1609'),
	('Deploy Container', 'execution', 'T1610'),
	('Exploitation for Client Execution', 'execution', 'T1203'),
	('Native API', 'execution', 'T1106'),
	('Serverless Execution', 'execution', 'T1648'),
	('Shared Modules', 'execution', 'T1129'),
	('Software Deployment Tools', 'execution', 'T1072'),
	('Windows Management Instrumentation', 'execution', 'T1047'),
]
for i in range(len(BASes_execution_childless)):
	if BASes_execution_childless[i][-1] in campaign_tactech["execution"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_execution_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_execution_childless[i], is_dummy=True)

AT.add_gate_logic("execution", "AND", color="#cc79a7", children=["execution_SystemServices","execution_ScheduledTaskJob","execution_CommandandScriptingInterpreter","execution_UserExecution","execution_InterProcessCommunication"]+[x[0] for x in BASes_execution_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: privilege-escalation

# Abuse Elevation Control Mechanism
BASes_privilegeescalation_AbuseElevationControlMechanism = [
	('Abuse Elevation Control Mechanism: Bypass User Account Control', 'privilege-escalation', 'T1548.002'),
	('Abuse Elevation Control Mechanism: Elevated Execution with Prompt', 'privilege-escalation', 'T1548.004'),
	('Abuse Elevation Control Mechanism: Setuid and Setgid', 'privilege-escalation', 'T1548.001'),
	('Abuse Elevation Control Mechanism: Sudo and Sudo Caching', 'privilege-escalation', 'T1548.003'),
	('Abuse Elevation Control Mechanism: Temporary Elevated Cloud Access', 'privilege-escalation', 'T1548.005'),
]
for i in range(len(BASes_privilegeescalation_AbuseElevationControlMechanism)):
	if BASes_privilegeescalation_AbuseElevationControlMechanism[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_AbuseElevationControlMechanism[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_AbuseElevationControlMechanism[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_AbuseElevationControlMechanism", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_AbuseElevationControlMechanism])

# Scheduled Task/Job
BASes_privilegeescalation_ScheduledTaskJob = [
	('Scheduled Task/Job: At', 'privilege-escalation', 'T1053.002'),
	('Scheduled Task/Job: Container Orchestration Job', 'privilege-escalation', 'T1053.007'),
	('Scheduled Task/Job: Cron', 'privilege-escalation', 'T1053.003'),
	('Scheduled Task/Job: Scheduled Task', 'privilege-escalation', 'T1053.005'),
	('Scheduled Task/Job: Systemd Timers', 'privilege-escalation', 'T1053.006'),
]
for i in range(len(BASes_privilegeescalation_ScheduledTaskJob)):
	if BASes_privilegeescalation_ScheduledTaskJob[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_ScheduledTaskJob[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_ScheduledTaskJob[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_ScheduledTaskJob", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_ScheduledTaskJob])

# Valid Accounts
BASes_privilegeescalation_ValidAccounts = [
	('Valid Accounts: Cloud Accounts', 'privilege-escalation', 'T1078.004'),
	('Valid Accounts: Default Accounts', 'privilege-escalation', 'T1078.001'),
	('Valid Accounts: Domain Accounts', 'privilege-escalation', 'T1078.002'),
	('Valid Accounts: Local Accounts', 'privilege-escalation', 'T1078.003'),
]
for i in range(len(BASes_privilegeescalation_ValidAccounts)):
	if BASes_privilegeescalation_ValidAccounts[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_ValidAccounts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_ValidAccounts[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_ValidAccounts", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_ValidAccounts])

# Account Manipulation
BASes_privilegeescalation_AccountManipulation = [
	('Account Manipulation: Additional Cloud Credentials', 'privilege-escalation', 'T1098.001'),
	('Account Manipulation: Additional Cloud Roles', 'privilege-escalation', 'T1098.003'),
	('Account Manipulation: Additional Container Cluster Roles', 'privilege-escalation', 'T1098.006'),
	('Account Manipulation: Additional Email Delegate Permissions', 'privilege-escalation', 'T1098.002'),
	('Account Manipulation: Device Registration', 'privilege-escalation', 'T1098.005'),
	('Account Manipulation: SSH Authorized Keys', 'privilege-escalation', 'T1098.004'),
]
for i in range(len(BASes_privilegeescalation_AccountManipulation)):
	if BASes_privilegeescalation_AccountManipulation[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_AccountManipulation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_AccountManipulation[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_AccountManipulation", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_AccountManipulation])

# Domain Policy Modification
BASes_privilegeescalation_DomainPolicyModification = [
	('Domain Policy Modification: Domain Trust Modification', 'privilege-escalation', 'T1484.002'),
	('Domain Policy Modification: Group Policy Modification', 'privilege-escalation', 'T1484.001'),
]
for i in range(len(BASes_privilegeescalation_DomainPolicyModification)):
	if BASes_privilegeescalation_DomainPolicyModification[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_DomainPolicyModification[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_DomainPolicyModification[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_DomainPolicyModification", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_DomainPolicyModification])

# Boot or Logon Initialization Scripts
BASes_privilegeescalation_BootorLogonInitializationScripts = [
	('Boot or Logon Initialization Scripts: Login Hook', 'privilege-escalation', 'T1037.002'),
	('Boot or Logon Initialization Scripts: Logon Script (Windows)', 'privilege-escalation', 'T1037.001'),
	('Boot or Logon Initialization Scripts: Network Logon Script', 'privilege-escalation', 'T1037.003'),
	('Boot or Logon Initialization Scripts: RC Scripts', 'privilege-escalation', 'T1037.004'),
	('Boot or Logon Initialization Scripts: Startup Items', 'privilege-escalation', 'T1037.005'),
]
for i in range(len(BASes_privilegeescalation_BootorLogonInitializationScripts)):
	if BASes_privilegeescalation_BootorLogonInitializationScripts[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_BootorLogonInitializationScripts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_BootorLogonInitializationScripts[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_BootorLogonInitializationScripts", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_BootorLogonInitializationScripts])

# Create or Modify System Process
BASes_privilegeescalation_CreateorModifySystemProcess = [
	('Create or Modify System Process: Launch Agent', 'privilege-escalation', 'T1543.001'),
	('Create or Modify System Process: Launch Daemon', 'privilege-escalation', 'T1543.004'),
	('Create or Modify System Process: Systemd Service', 'privilege-escalation', 'T1543.002'),
	('Create or Modify System Process: Windows Service', 'privilege-escalation', 'T1543.003'),
]
for i in range(len(BASes_privilegeescalation_CreateorModifySystemProcess)):
	if BASes_privilegeescalation_CreateorModifySystemProcess[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_CreateorModifySystemProcess[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_CreateorModifySystemProcess[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_CreateorModifySystemProcess", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_CreateorModifySystemProcess])

# Event Triggered Execution
BASes_privilegeescalation_EventTriggeredExecution = [
	('Event Triggered Execution: Accessibility Features', 'privilege-escalation', 'T1546.008'),
	('Event Triggered Execution: AppCert DLLs', 'privilege-escalation', 'T1546.009'),
	('Event Triggered Execution: AppInit DLLs', 'privilege-escalation', 'T1546.010'),
	('Event Triggered Execution: Application Shimming', 'privilege-escalation', 'T1546.011'),
	('Event Triggered Execution: Change Default File Association', 'privilege-escalation', 'T1546.001'),
	('Event Triggered Execution: Component Object Model Hijacking', 'privilege-escalation', 'T1546.015'),
	('Event Triggered Execution: Emond', 'privilege-escalation', 'T1546.014'),
	('Event Triggered Execution: Image File Execution Options Injection', 'privilege-escalation', 'T1546.012'),
	('Event Triggered Execution: Installer Packages', 'privilege-escalation', 'T1546.016'),
	('Event Triggered Execution: LC_LOAD_DYLIB Addition', 'privilege-escalation', 'T1546.006'),
	('Event Triggered Execution: Netsh Helper DLL', 'privilege-escalation', 'T1546.007'),
	('Event Triggered Execution: PowerShell Profile', 'privilege-escalation', 'T1546.013'),
	('Event Triggered Execution: Screensaver', 'privilege-escalation', 'T1546.002'),
	('Event Triggered Execution: Trap', 'privilege-escalation', 'T1546.005'),
	('Event Triggered Execution: Unix Shell Configuration Modification', 'privilege-escalation', 'T1546.004'),
	('Event Triggered Execution: Windows Management Instrumentation Event Subscription', 'privilege-escalation', 'T1546.003'),
]
for i in range(len(BASes_privilegeescalation_EventTriggeredExecution)):
	if BASes_privilegeescalation_EventTriggeredExecution[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_EventTriggeredExecution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_EventTriggeredExecution[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_EventTriggeredExecution", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_EventTriggeredExecution])

# Hijack Execution Flow
BASes_privilegeescalation_HijackExecutionFlow = [
	('Hijack Execution Flow: COR_PROFILER', 'privilege-escalation', 'T1574.012'),
	('Hijack Execution Flow: DLL Search Order Hijacking', 'privilege-escalation', 'T1574.001'),
	('Hijack Execution Flow: DLL Side-Loading', 'privilege-escalation', 'T1574.002'),
	('Hijack Execution Flow: Dylib Hijacking', 'privilege-escalation', 'T1574.004'),
	('Hijack Execution Flow: Dynamic Linker Hijacking', 'privilege-escalation', 'T1574.006'),
	('Hijack Execution Flow: Executable Installer File Permissions Weakness', 'privilege-escalation', 'T1574.005'),
	('Hijack Execution Flow: KernelCallbackTable', 'privilege-escalation', 'T1574.013'),
	('Hijack Execution Flow: Path Interception by PATH Environment Variable', 'privilege-escalation', 'T1574.007'),
	('Hijack Execution Flow: Path Interception by Search Order Hijacking', 'privilege-escalation', 'T1574.008'),
	('Hijack Execution Flow: Path Interception by Unquoted Path', 'privilege-escalation', 'T1574.009'),
	('Hijack Execution Flow: Services File Permissions Weakness', 'privilege-escalation', 'T1574.010'),
	('Hijack Execution Flow: Services Registry Permissions Weakness', 'privilege-escalation', 'T1574.011'),
]
for i in range(len(BASes_privilegeescalation_HijackExecutionFlow)):
	if BASes_privilegeescalation_HijackExecutionFlow[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_HijackExecutionFlow[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_HijackExecutionFlow[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_HijackExecutionFlow", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_HijackExecutionFlow])

# Boot or Logon Autostart Execution
BASes_privilegeescalation_BootorLogonAutostartExecution = [
	('Boot or Logon Autostart Execution: Active Setup', 'privilege-escalation', 'T1547.014'),
	('Boot or Logon Autostart Execution: Authentication Package', 'privilege-escalation', 'T1547.002'),
	('Boot or Logon Autostart Execution: Kernel Modules and Extensions', 'privilege-escalation', 'T1547.006'),
	('Boot or Logon Autostart Execution: LSASS Driver', 'privilege-escalation', 'T1547.008'),
	('Boot or Logon Autostart Execution: Login Items', 'privilege-escalation', 'T1547.015'),
	('Boot or Logon Autostart Execution: Port Monitors', 'privilege-escalation', 'T1547.010'),
	('Boot or Logon Autostart Execution: Print Processors', 'privilege-escalation', 'T1547.012'),
	('Boot or Logon Autostart Execution: Re-opened Applications', 'privilege-escalation', 'T1547.007'),
	('Boot or Logon Autostart Execution: Registry Run Keys / Startup Folder', 'privilege-escalation', 'T1547.001'),
	('Boot or Logon Autostart Execution: Security Support Provider', 'privilege-escalation', 'T1547.005'),
	('Boot or Logon Autostart Execution: Shortcut Modification', 'privilege-escalation', 'T1547.009'),
	('Boot or Logon Autostart Execution: Time Providers', 'privilege-escalation', 'T1547.003'),
	('Boot or Logon Autostart Execution: Winlogon Helper DLL', 'privilege-escalation', 'T1547.004'),
	('Boot or Logon Autostart Execution: XDG Autostart Entries', 'privilege-escalation', 'T1547.013'),
]
for i in range(len(BASes_privilegeescalation_BootorLogonAutostartExecution)):
	if BASes_privilegeescalation_BootorLogonAutostartExecution[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_BootorLogonAutostartExecution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_BootorLogonAutostartExecution[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_BootorLogonAutostartExecution", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_BootorLogonAutostartExecution])

# Access Token Manipulation
BASes_privilegeescalation_AccessTokenManipulation = [
	('Access Token Manipulation: Create Process with Token', 'privilege-escalation', 'T1134.002'),
	('Access Token Manipulation: Make and Impersonate Token', 'privilege-escalation', 'T1134.003'),
	('Access Token Manipulation: Parent PID Spoofing', 'privilege-escalation', 'T1134.004'),
	('Access Token Manipulation: SID-History Injection', 'privilege-escalation', 'T1134.005'),
	('Access Token Manipulation: Token Impersonation/Theft', 'privilege-escalation', 'T1134.001'),
]
for i in range(len(BASes_privilegeescalation_AccessTokenManipulation)):
	if BASes_privilegeescalation_AccessTokenManipulation[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_AccessTokenManipulation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_AccessTokenManipulation[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_AccessTokenManipulation", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_AccessTokenManipulation])

# Process Injection
BASes_privilegeescalation_ProcessInjection = [
	('Process Injection: Asynchronous Procedure Call', 'privilege-escalation', 'T1055.004'),
	('Process Injection: Dynamic-link Library Injection', 'privilege-escalation', 'T1055.001'),
	('Process Injection: Extra Window Memory Injection', 'privilege-escalation', 'T1055.011'),
	('Process Injection: ListPlanting', 'privilege-escalation', 'T1055.015'),
	('Process Injection: Portable Executable Injection', 'privilege-escalation', 'T1055.002'),
	('Process Injection: Proc Memory', 'privilege-escalation', 'T1055.009'),
	('Process Injection: Process Doppelgänging', 'privilege-escalation', 'T1055.013'),
	('Process Injection: Process Hollowing', 'privilege-escalation', 'T1055.012'),
	('Process Injection: Ptrace System Calls', 'privilege-escalation', 'T1055.008'),
	('Process Injection: Thread Execution Hijacking', 'privilege-escalation', 'T1055.003'),
	('Process Injection: Thread Local Storage', 'privilege-escalation', 'T1055.005'),
	('Process Injection: VDSO Hijacking', 'privilege-escalation', 'T1055.014'),
]
for i in range(len(BASes_privilegeescalation_ProcessInjection)):
	if BASes_privilegeescalation_ProcessInjection[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_ProcessInjection[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_ProcessInjection[i], is_dummy=True)

AT.add_gate_logic("privilegeescalation_ProcessInjection", "OR", color="#88ccee", children=[ x[0] for x in BASes_privilegeescalation_ProcessInjection])

# direct BASes
BASes_privilegeescalation_childless = [
	('Escape to Host', 'privilege-escalation', 'T1611'),
	('Exploitation for Privilege Escalation', 'privilege-escalation', 'T1068'),
]
for i in range(len(BASes_privilegeescalation_childless)):
	if BASes_privilegeescalation_childless[i][-1] in campaign_tactech["privilege-escalation"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_privilegeescalation_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_privilegeescalation_childless[i], is_dummy=True)

AT.add_gate_logic("privilege-escalation", "AND", color="#cc79a7", children=["privilegeescalation_AbuseElevationControlMechanism","privilegeescalation_ScheduledTaskJob","privilegeescalation_ValidAccounts","privilegeescalation_AccountManipulation","privilegeescalation_DomainPolicyModification","privilegeescalation_BootorLogonInitializationScripts","privilegeescalation_CreateorModifySystemProcess","privilegeescalation_EventTriggeredExecution","privilegeescalation_HijackExecutionFlow","privilegeescalation_BootorLogonAutostartExecution","privilegeescalation_AccessTokenManipulation","privilegeescalation_ProcessInjection"]+[x[0] for x in BASes_privilegeescalation_childless])


# - - - - - - - - - - - - - - - - - -
# Tactic: persistence

# Office Application Startup
BASes_persistence_OfficeApplicationStartup = [
	('Office Application Startup: Add-ins', 'persistence', 'T1137.006'),
	('Office Application Startup: Office Template Macros', 'persistence', 'T1137.001'),
	('Office Application Startup: Office Test', 'persistence', 'T1137.002'),
	('Office Application Startup: Outlook Forms', 'persistence', 'T1137.003'),
	('Office Application Startup: Outlook Home Page', 'persistence', 'T1137.004'),
	('Office Application Startup: Outlook Rules', 'persistence', 'T1137.005'),
]
for i in range(len(BASes_persistence_OfficeApplicationStartup)):
	if BASes_persistence_OfficeApplicationStartup[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_OfficeApplicationStartup[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_OfficeApplicationStartup[i], is_dummy=True)

AT.add_gate_logic("persistence_OfficeApplicationStartup", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_OfficeApplicationStartup])

# Modify Authentication Process
BASes_persistence_ModifyAuthenticationProcess = [
	('Modify Authentication Process: Domain Controller Authentication', 'persistence', 'T1556.001'),
	('Modify Authentication Process: Hybrid Identity', 'persistence', 'T1556.007'),
	('Modify Authentication Process: Multi-Factor Authentication', 'persistence', 'T1556.006'),
	('Modify Authentication Process: Network Device Authentication', 'persistence', 'T1556.004'),
	('Modify Authentication Process: Network Provider DLL', 'persistence', 'T1556.008'),
	('Modify Authentication Process: Password Filter DLL', 'persistence', 'T1556.002'),
	('Modify Authentication Process: Pluggable Authentication Modules', 'persistence', 'T1556.003'),
	('Modify Authentication Process: Reversible Encryption', 'persistence', 'T1556.005'),
]
for i in range(len(BASes_persistence_ModifyAuthenticationProcess)):
	if BASes_persistence_ModifyAuthenticationProcess[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_ModifyAuthenticationProcess[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_ModifyAuthenticationProcess[i], is_dummy=True)

AT.add_gate_logic("persistence_ModifyAuthenticationProcess", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_ModifyAuthenticationProcess])

# Scheduled Task/Job
BASes_persistence_ScheduledTaskJob = [
	('Scheduled Task/Job: At', 'persistence', 'T1053.002'),
	('Scheduled Task/Job: Container Orchestration Job', 'persistence', 'T1053.007'),
	('Scheduled Task/Job: Cron', 'persistence', 'T1053.003'),
	('Scheduled Task/Job: Scheduled Task', 'persistence', 'T1053.005'),
	('Scheduled Task/Job: Systemd Timers', 'persistence', 'T1053.006'),
]
for i in range(len(BASes_persistence_ScheduledTaskJob)):
	if BASes_persistence_ScheduledTaskJob[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_ScheduledTaskJob[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_ScheduledTaskJob[i], is_dummy=True)

AT.add_gate_logic("persistence_ScheduledTaskJob", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_ScheduledTaskJob])

# Account Manipulation
BASes_persistence_AccountManipulation = [
	('Account Manipulation: Additional Cloud Credentials', 'persistence', 'T1098.001'),
	('Account Manipulation: Additional Cloud Roles', 'persistence', 'T1098.003'),
	('Account Manipulation: Additional Container Cluster Roles', 'persistence', 'T1098.006'),
	('Account Manipulation: Additional Email Delegate Permissions', 'persistence', 'T1098.002'),
	('Account Manipulation: Device Registration', 'persistence', 'T1098.005'),
	('Account Manipulation: SSH Authorized Keys', 'persistence', 'T1098.004'),
]
for i in range(len(BASes_persistence_AccountManipulation)):
	if BASes_persistence_AccountManipulation[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_AccountManipulation[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_AccountManipulation[i], is_dummy=True)

AT.add_gate_logic("persistence_AccountManipulation", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_AccountManipulation])

# Valid Accounts
BASes_persistence_ValidAccounts = [
	('Valid Accounts: Cloud Accounts', 'persistence', 'T1078.004'),
	('Valid Accounts: Default Accounts', 'persistence', 'T1078.001'),
	('Valid Accounts: Domain Accounts', 'persistence', 'T1078.002'),
	('Valid Accounts: Local Accounts', 'persistence', 'T1078.003'),
]
for i in range(len(BASes_persistence_ValidAccounts)):
	if BASes_persistence_ValidAccounts[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_ValidAccounts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_ValidAccounts[i], is_dummy=True)

AT.add_gate_logic("persistence_ValidAccounts", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_ValidAccounts])

# Boot or Logon Initialization Scripts
BASes_persistence_BootorLogonInitializationScripts = [
	('Boot or Logon Initialization Scripts: Login Hook', 'persistence', 'T1037.002'),
	('Boot or Logon Initialization Scripts: Logon Script (Windows)', 'persistence', 'T1037.001'),
	('Boot or Logon Initialization Scripts: Network Logon Script', 'persistence', 'T1037.003'),
	('Boot or Logon Initialization Scripts: RC Scripts', 'persistence', 'T1037.004'),
	('Boot or Logon Initialization Scripts: Startup Items', 'persistence', 'T1037.005'),
]
for i in range(len(BASes_persistence_BootorLogonInitializationScripts)):
	if BASes_persistence_BootorLogonInitializationScripts[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_BootorLogonInitializationScripts[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_BootorLogonInitializationScripts[i], is_dummy=True)

AT.add_gate_logic("persistence_BootorLogonInitializationScripts", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_BootorLogonInitializationScripts])

# Create or Modify System Process
BASes_persistence_CreateorModifySystemProcess = [
	('Create or Modify System Process: Launch Agent', 'persistence', 'T1543.001'),
	('Create or Modify System Process: Launch Daemon', 'persistence', 'T1543.004'),
	('Create or Modify System Process: Systemd Service', 'persistence', 'T1543.002'),
	('Create or Modify System Process: Windows Service', 'persistence', 'T1543.003'),
]
for i in range(len(BASes_persistence_CreateorModifySystemProcess)):
	if BASes_persistence_CreateorModifySystemProcess[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_CreateorModifySystemProcess[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_CreateorModifySystemProcess[i], is_dummy=True)

AT.add_gate_logic("persistence_CreateorModifySystemProcess", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_CreateorModifySystemProcess])

# Create Account
BASes_persistence_CreateAccount = [
	('Create Account: Cloud Account', 'persistence', 'T1136.003'),
	('Create Account: Domain Account', 'persistence', 'T1136.002'),
	('Create Account: Local Account', 'persistence', 'T1136.001'),
]
for i in range(len(BASes_persistence_CreateAccount)):
	if BASes_persistence_CreateAccount[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_CreateAccount[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_CreateAccount[i], is_dummy=True)

AT.add_gate_logic("persistence_CreateAccount", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_CreateAccount])

# Event Triggered Execution
BASes_persistence_EventTriggeredExecution = [
	('Event Triggered Execution: Accessibility Features', 'persistence', 'T1546.008'),
	('Event Triggered Execution: AppCert DLLs', 'persistence', 'T1546.009'),
	('Event Triggered Execution: AppInit DLLs', 'persistence', 'T1546.010'),
	('Event Triggered Execution: Application Shimming', 'persistence', 'T1546.011'),
	('Event Triggered Execution: Change Default File Association', 'persistence', 'T1546.001'),
	('Event Triggered Execution: Component Object Model Hijacking', 'persistence', 'T1546.015'),
	('Event Triggered Execution: Emond', 'persistence', 'T1546.014'),
	('Event Triggered Execution: Image File Execution Options Injection', 'persistence', 'T1546.012'),
	('Event Triggered Execution: Installer Packages', 'persistence', 'T1546.016'),
	('Event Triggered Execution: LC_LOAD_DYLIB Addition', 'persistence', 'T1546.006'),
	('Event Triggered Execution: Netsh Helper DLL', 'persistence', 'T1546.007'),
	('Event Triggered Execution: PowerShell Profile', 'persistence', 'T1546.013'),
	('Event Triggered Execution: Screensaver', 'persistence', 'T1546.002'),
	('Event Triggered Execution: Trap', 'persistence', 'T1546.005'),
	('Event Triggered Execution: Unix Shell Configuration Modification', 'persistence', 'T1546.004'),
	('Event Triggered Execution: Windows Management Instrumentation Event Subscription', 'persistence', 'T1546.003'),
]
for i in range(len(BASes_persistence_EventTriggeredExecution)):
	if BASes_persistence_EventTriggeredExecution[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_EventTriggeredExecution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_EventTriggeredExecution[i], is_dummy=True)

AT.add_gate_logic("persistence_EventTriggeredExecution", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_EventTriggeredExecution])

# Hijack Execution Flow
BASes_persistence_HijackExecutionFlow = [
	('Hijack Execution Flow: COR_PROFILER', 'persistence', 'T1574.012'),
	('Hijack Execution Flow: DLL Search Order Hijacking', 'persistence', 'T1574.001'),
	('Hijack Execution Flow: DLL Side-Loading', 'persistence', 'T1574.002'),
	('Hijack Execution Flow: Dylib Hijacking', 'persistence', 'T1574.004'),
	('Hijack Execution Flow: Dynamic Linker Hijacking', 'persistence', 'T1574.006'),
	('Hijack Execution Flow: Executable Installer File Permissions Weakness', 'persistence', 'T1574.005'),
	('Hijack Execution Flow: KernelCallbackTable', 'persistence', 'T1574.013'),
	('Hijack Execution Flow: Path Interception by PATH Environment Variable', 'persistence', 'T1574.007'),
	('Hijack Execution Flow: Path Interception by Search Order Hijacking', 'persistence', 'T1574.008'),
	('Hijack Execution Flow: Path Interception by Unquoted Path', 'persistence', 'T1574.009'),
	('Hijack Execution Flow: Services File Permissions Weakness', 'persistence', 'T1574.010'),
	('Hijack Execution Flow: Services Registry Permissions Weakness', 'persistence', 'T1574.011'),
]
for i in range(len(BASes_persistence_HijackExecutionFlow)):
	if BASes_persistence_HijackExecutionFlow[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_HijackExecutionFlow[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_HijackExecutionFlow[i], is_dummy=True)

AT.add_gate_logic("persistence_HijackExecutionFlow", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_HijackExecutionFlow])

# Traffic Signaling
BASes_persistence_TrafficSignaling = [
	('Traffic Signaling: Port Knocking', 'persistence', 'T1205.001'),
	('Traffic Signaling: Socket Filters', 'persistence', 'T1205.002'),
]
for i in range(len(BASes_persistence_TrafficSignaling)):
	if BASes_persistence_TrafficSignaling[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_TrafficSignaling[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_TrafficSignaling[i], is_dummy=True)

AT.add_gate_logic("persistence_TrafficSignaling", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_TrafficSignaling])

# Boot or Logon Autostart Execution
BASes_persistence_BootorLogonAutostartExecution = [
	('Boot or Logon Autostart Execution: Active Setup', 'persistence', 'T1547.014'),
	('Boot or Logon Autostart Execution: Authentication Package', 'persistence', 'T1547.002'),
	('Boot or Logon Autostart Execution: Kernel Modules and Extensions', 'persistence', 'T1547.006'),
	('Boot or Logon Autostart Execution: LSASS Driver', 'persistence', 'T1547.008'),
	('Boot or Logon Autostart Execution: Login Items', 'persistence', 'T1547.015'),
	('Boot or Logon Autostart Execution: Port Monitors', 'persistence', 'T1547.010'),
	('Boot or Logon Autostart Execution: Print Processors', 'persistence', 'T1547.012'),
	('Boot or Logon Autostart Execution: Re-opened Applications', 'persistence', 'T1547.007'),
	('Boot or Logon Autostart Execution: Registry Run Keys / Startup Folder', 'persistence', 'T1547.001'),
	('Boot or Logon Autostart Execution: Security Support Provider', 'persistence', 'T1547.005'),
	('Boot or Logon Autostart Execution: Shortcut Modification', 'persistence', 'T1547.009'),
	('Boot or Logon Autostart Execution: Time Providers', 'persistence', 'T1547.003'),
	('Boot or Logon Autostart Execution: Winlogon Helper DLL', 'persistence', 'T1547.004'),
	('Boot or Logon Autostart Execution: XDG Autostart Entries', 'persistence', 'T1547.013'),
]
for i in range(len(BASes_persistence_BootorLogonAutostartExecution)):
	if BASes_persistence_BootorLogonAutostartExecution[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_BootorLogonAutostartExecution[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_BootorLogonAutostartExecution[i], is_dummy=True)

AT.add_gate_logic("persistence_BootorLogonAutostartExecution", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_BootorLogonAutostartExecution])

# Pre-OS Boot
BASes_persistence_PreOSBoot = [
	('Pre-OS Boot: Bootkit', 'persistence', 'T1542.003'),
	('Pre-OS Boot: Component Firmware', 'persistence', 'T1542.002'),
	('Pre-OS Boot: ROMMONkit', 'persistence', 'T1542.004'),
	('Pre-OS Boot: System Firmware', 'persistence', 'T1542.001'),
	('Pre-OS Boot: TFTP Boot', 'persistence', 'T1542.005'),
]
for i in range(len(BASes_persistence_PreOSBoot)):
	if BASes_persistence_PreOSBoot[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_PreOSBoot[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_PreOSBoot[i], is_dummy=True)

AT.add_gate_logic("persistence_PreOSBoot", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_PreOSBoot])

# Server Software Component
BASes_persistence_ServerSoftwareComponent = [
	('Server Software Component: IIS Components', 'persistence', 'T1505.004'),
	('Server Software Component: SQL Stored Procedures', 'persistence', 'T1505.001'),
	('Server Software Component: Terminal Services DLL', 'persistence', 'T1505.005'),
	('Server Software Component: Transport Agent', 'persistence', 'T1505.002'),
	('Server Software Component: Web Shell', 'persistence', 'T1505.003'),
]
for i in range(len(BASes_persistence_ServerSoftwareComponent)):
	if BASes_persistence_ServerSoftwareComponent[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_ServerSoftwareComponent[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_ServerSoftwareComponent[i], is_dummy=True)

AT.add_gate_logic("persistence_ServerSoftwareComponent", "OR", color="#88ccee", children=[ x[0] for x in BASes_persistence_ServerSoftwareComponent])

# direct BASes
BASes_persistence_childless = [
	('BITS Jobs', 'persistence', 'T1197'),
	('Browser Extensions', 'persistence', 'T1176'),
	('Compromise Client Software Binary', 'persistence', 'T1554'),
	('External Remote Services', 'persistence', 'T1133'),
	('Implant Internal Image', 'persistence', 'T1525'),
	('Power Settings', 'persistence', 'T1653'),
]
for i in range(len(BASes_persistence_childless)):
	if BASes_persistence_childless[i][-1] in campaign_tactech["persistence"]:
		# This is a technique from our campaign (or we're generating the full MITRE AT template)
		AT.add_BAS_MITRE(*BASes_persistence_childless[i])
	else:
		# This is a technique outside our campaign
		AT.add_BAS_MITRE(*BASes_persistence_childless[i], is_dummy=True)

AT.add_gate_logic("persistence", "AND", color="#cc79a7", children=["persistence_OfficeApplicationStartup","persistence_ModifyAuthenticationProcess","persistence_ScheduledTaskJob","persistence_AccountManipulation","persistence_ValidAccounts","persistence_BootorLogonInitializationScripts","persistence_CreateorModifySystemProcess","persistence_CreateAccount","persistence_EventTriggeredExecution","persistence_HijackExecutionFlow","persistence_TrafficSignaling","persistence_BootorLogonAutostartExecution","persistence_PreOSBoot","persistence_ServerSoftwareComponent"]+[x[0] for x in BASes_persistence_childless])

# TLE connecting all tactics

AT.add_gate_logic("glorrrious_TLE", "SAND", color="#cc79a7", children = ["collection","command-and-control","reconnaissance","discovery","initial-access","resource-development","lateral-movement","credential-access","impact","exfiltration","defense-evasion","execution","privilege-escalation","persistence"])

assert (AT.is_well_defined(verbose=True))
full_AT = AT.evil_twin(verbose=True)
assert (full_AT.is_static(verbose=True))
assert (full_AT.is_well_defined(verbose=True))

print("Computing AT probability via cATM...")
print("AT cATM: " + str(full_AT.compute_metric("cATM", verbose=True)))
AT.show(size=(111,111), fname="MATT_default_C0014.pdf")
